/* RPC extension for IP connection tracking, Version 2.2
 * (C) 2000 by Marcelo Barbosa Lima <marcelo.lima@dcc.unicamp.br>
 *      - original rpc tracking module
 *      - "recent" connection handling for kernel 2.3+ netfilter
 *
 * (C) 2001 by Rusty Russell <rusty@rustcorp.com.au>
 *      - upgraded conntrack modules to oldnat api - kernel 2.4.0+
 *
 * (C) 2002 by Ian (Larry) Latter <Ian.Latter@mq.edu.au>
 *      - upgraded conntrack modules to newnat api - kernel 2.4.20+
 *      - extended matching to support filtering on procedures
 *
 * (c) 2008 by Franz Regensburger (FABR) <franz.regensburger@odorf.de>
 *      - changed definition of structure request_p in order to reflect 
 *          added tracking support for
 *        - rpc program 100000 (portmapper) procedure DUMP
 *        - proprietary RPC based negotiation of server ports for Clear Case
 *          (albd daemon)
 *
 * ip_conntrack_rpc.h,v 2.2 2003/01/12 18:30:00
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License
 *      as published by the Free Software Foundation; either version
 *      2 of the License, or (at your option) any later version.
 **
 */

#include <asm/param.h>
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/stddef.h>
#include <linux/list.h>

#include <linux/netfilter_ipv4/ip_conntrack_helper.h>

#ifndef _IP_CONNTRACK_RPC_H
#define _IP_CONNTRACK_RPC_H

#define RPC_PORT  111

#define RPC_PKG_NOMATCH   0                 /* FABR: for the sake of a better documentation */
#define RPC_PKG_MATCH     1                 /* FABR: for the sake of a better documentation */

#define RPC_CONNTRACK_DUMP_MAX_EXPECTED 10  /* FABR: maximal number of simultaneously expected
                                             * ports due to a DUMP reply
                                             */

/* Datum in RPC packets are encoded in XDR */
#define IXDR_GET_INT32(buf) ((u_int32_t) ntohl((uint32_t)*buf))

/* Fast timeout, to deny DoS atacks */
#define EXP (60 * HZ)

/* Normal timeouts */
#define EXPIRES (180 * HZ)

/* For future conections RPC, using client's cache bindings
 * I'll use ip_conntrack_lock to lock these lists       */

/* FABR: Bit masks for the encoding of slot blob in struct request_p.
 *
 * We store some additional data in a compressed form using a u_int_16 blob.
 *
 */
#define REQUEST_P_BLOB_KIND_MASK         0xF000
#define REQUEST_P_BLOB_KIND_GETPORT      0x8000
#define REQUEST_P_BLOB_KIND_GETPORT_ALBD 0x4000

#define REQUEST_P_BLOB_SNDCH_MASK        0x0F00
#define REQUEST_P_BLOB_SNDCH_YES         0x0800
#define REQUEST_P_BLOB_SNDCH_NO          0x0000

/* the rest of the u_int_16 blob is currently unused */

/* This identifies each request and stores the requested protocol
 * along with some additional blob data
 */
struct request_p {
    struct list_head list;

    /* Used as search key */
    u_int32_t xid;   /* XID of RPC call */   
    u_int32_t ip;    /* IP src address of RPC caller */

    /* Rest */
    u_int16_t blob; /* A blob with compressed data.                         */
                    /* See the REQUEST_P_BLOB_*definitions above            */

    u_int16_t proto;/* Protocol number inside the GETPORT procedure request */

    struct timer_list timeout;
};


/* Use xid+ip for normal equality test; this is no longer used */
static inline int request_p_cmp_normal(const struct request_p *p,
                                       u_int32_t xid, 
                                       u_int32_t ip) {
    return (p->xid == xid && p->ip == ip);
}


/* Use xid+ip+kind where kind==(GETPORT|GETPORT_ALBD) for equality test
 * This ignores all entries of a different kind.
 */
static inline int request_p_cmp_kind_getport(const struct request_p *p,
                                     u_int32_t xid, 
                                     u_int32_t ip
                                     ) {
    return (p->xid == xid && p->ip == ip &&
            (
                (p->blob & REQUEST_P_BLOB_KIND_MASK) == REQUEST_P_BLOB_KIND_GETPORT ||
                (p->blob & REQUEST_P_BLOB_KIND_MASK) == REQUEST_P_BLOB_KIND_GETPORT_ALBD
                )
        );
}

#endif /* _IP_CONNTRACK_RPC_H */
