/* RPC extension for IP (TCP) connection tracking, Version 2.2
 * (C) 2000 by Marcelo Barbosa Lima <marcelo.lima@dcc.unicamp.br>
 *      - original rpc tracking module
 *      - "recent" connection handling for kernel 2.3+ netfilter
 *
 * (C) 2001 by Rusty Russell <rusty@rustcorp.com.au>
 *      - upgraded conntrack modules to oldnat api - kernel 2.4.0+
 *
 * (C) 2002,2003 by Ian (Larry) Latter <Ian.Latter@mq.edu.au>
 *      - upgraded conntrack modules to newnat api - kernel 2.4.20+
 *      - extended matching to support filtering on procedures
 *
 * (c) 2004,2005 by David Stes <stes@pandora.be>
 *      - add nsrexec option for Legato NetWorker
 *      - upgraded to 2.6.12+ conntrack module api
 *
 * (c) 2008 by Franz Regensburger (FABR) <franz.regensburger@odorf.de>
 *      - added tracking support for
 *        - rpc program 100000 (portmapper) procedure DUMP
 *        - proprietary RPC based negotiation of server ports for Clear Case (albd daemon)
 *
 * ip_conntrack_rpc_tpc.c,v 2.2 2003/01/12 18:30:00
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License
 *      as published by the Free Software Foundation; either version
 *      2 of the License, or (at your option) any later version.
 **
 *      Module load syntax:
 *
 *      insmod ip_conntrack_rpc_tcp.o nsrexec=<n> ports=port1,...port<MAX_PORTS>
 *
 *      Please give the ports of all RPC servers you wish to connect to.
 *      For example, ports=111,371,7938 where
 *         111 is the standard portmapper port
 *         371 is for the ClearCase ALBD portmapper 
 *        7938 is for Legato NetWorker's portmapper
 *
 *      If you don't specify ports, the default will be port 111 (SUN portmap).
 *
 *      If you like to track the Legato Networker traffic you also have to specify
 *      the port of the rexec() service of Legato NetWorker.  For example, nsrexec=7937
 **
 *      Note to all:
 *
 *      RPCs should not be exposed to the internet - ask the Pentagon;
 *
 *        "The unidentified crackers pleaded guilty in July to charges
 *         of juvenile delinquency stemming from a string of Pentagon
 *         network intrusions in February.
 *
 *         The youths, going by the names TooShort and Makaveli, used
 *         a common server security hole to break in, according to
 *         Dane Jasper, owner of the California Internet service
 *         provider, Sonic. They used the hole, known as the 'statd'
 *         exploit, to attempt more than 800 break-ins, Jasper said."
 *
 *      From: Wired News; "Pentagon Kids Kicked Off Grid" - Nov 6, 1998
 *      URL:  http://www.wired.com/news/politics/0,1283,16098,00.html
 **
 */

#include <linux/module.h>
#include <linux/netfilter.h>
#include <linux/ip.h>
#include <net/checksum.h>
#include <net/tcp.h>

#include <asm/param.h>
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/stddef.h>
#include <linux/list.h>

#include <linux/netfilter_ipv4/lockhelp.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ip_conntrack_helper.h>
#include <linux/netfilter_ipv4/ip_conntrack_rpc.h>

#define MAX_PORTS 8
static int ports[MAX_PORTS];
static int ports_n_c = 0;
static int nsrexec = 0;

#ifdef MODULE_PARM
module_param(nsrexec,int, 0400);
MODULE_PARM_DESC(nsrexec, "TCP port of Legato NetWorker's rexec service");
module_param_array(ports, int, &ports_n_c, 0400);
MODULE_PARM_DESC(ports, "port numbers (TCP/UDP) of RPC portmapper servers");
#endif

MODULE_AUTHOR("Marcelo Barbosa Lima <marcelo.lima@dcc.unicamp.br>");
MODULE_DESCRIPTION("RPC TCP connection tracking module");
MODULE_LICENSE("GPL");

/* Here we define the amount of output we like.
 * Use define or undef according to your preferences.
 */

#define RPC_DO_PINFO
#define RPC_DO_PERROR
#define RPC_DO_PWARN
#undef  RPC_DO_PDEBUG_L1
#undef  RPC_DO_PDEBUG_L2

/* And here we define the output functions depending on the above settings */ 
#define RPC_PROGNAME "ip_conntrack_rpc_tcp: "
#ifdef  RPC_DO_PINFO
#define RPC_PINFO(format, args...) printk(KERN_DEBUG RPC_PROGNAME format, ## args)
#else
#define RPC_PINFO(format, args...)
#endif

#ifdef  RPC_DO_PERROR
#define RPC_PERROR(format, args...) printk(KERN_DEBUG RPC_PROGNAME format, ## args)
#else
#define RPC_PERROR(format, args...)
#endif

#ifdef  RPC_DO_PWARN
#define RPC_PWARN(format, args...) printk(KERN_DEBUG RPC_PROGNAME format, ## args)
#else
#define RPC_PWARN(format, args...)
#endif

#ifdef  RPC_DO_PDEBUG_L1
#define RPC_PDEBUG_L1(format, args...) printk(KERN_DEBUG RPC_PROGNAME format, ## args)
#else
#define RPC_PDEBUG_L1(format, args...)
#endif

#ifdef  RPC_DO_PDEBUG_L2
#define RPC_PDEBUG_L2(format, args...) printk(KERN_DEBUG RPC_PROGNAME format, ## args)
#else
#define RPC_PDEBUG_L2(format, args...)
#endif

DECLARE_RWLOCK(ipct_rpc_tcp_lock);
#define ASSERT_READ_LOCK(x) MUST_BE_READ_LOCKED(&ipct_rpc_tcp_lock)
#define ASSERT_WRITE_LOCK(x) MUST_BE_WRITE_LOCKED(&ipct_rpc_tcp_lock)
#include <linux/netfilter_ipv4/listhelp.h>

/* For future conections RPC, using client's cache bindings
 * I'll use ip_conntrack_lock to lock these lists
 */

/* FABR NOTE: the list manipulation functions/macros are defined in
 * <linux kernel source>/include/linux/netfilter_ipv4/listhelp.h
 * <linux kernel source>/include/linux/list.h
 */

LIST_HEAD(request_p_list_tcp);


static void delete_request_p(unsigned long request_p_ul)
{
  struct request_p *p = (void *)request_p_ul;
        
  WRITE_LOCK(&ipct_rpc_tcp_lock);
  LIST_DELETE(&request_p_list_tcp, p);
  WRITE_UNLOCK(&ipct_rpc_tcp_lock);
  kfree(p);
  return;
}


static void req_cl(struct request_p * r)
{
  WRITE_LOCK(&ipct_rpc_tcp_lock);
  del_timer(&r->timeout);
  LIST_DELETE(&request_p_list_tcp, r);
  WRITE_UNLOCK(&ipct_rpc_tcp_lock);
  kfree(r);
  return;
}


static void clean_request(struct list_head *list)
{
  struct list_head *first = list->prev;
  struct list_head *temp = list->next;
  struct list_head *aux;

  if (list_empty(list))
    return;

  while (first != temp) {
    aux = temp->next;
    req_cl((struct request_p *)temp);
    temp = aux;     
  }
  req_cl((struct request_p *)temp);
  return;
}


static void alloc_request_p_tcp(u_int32_t xid,  u_int32_t ip,
                                u_int16_t blob, u_int16_t proto)
{
  struct request_p *req_p;
        
  /* If an entry already exists we just refresh the timer */
  WRITE_LOCK(&ipct_rpc_tcp_lock);
  req_p = LIST_FIND(&request_p_list_tcp, request_p_cmp_kind_getport,
		    struct request_p *, xid, ip);

  if (req_p) {
    /* Refresh timeout */
    if (del_timer(&req_p->timeout)) {
      req_p->timeout.expires = jiffies + EXP;
      add_timer(&req_p->timeout);
    }
    WRITE_UNLOCK(&ipct_rpc_tcp_lock);
    return; 

  }
  WRITE_UNLOCK(&ipct_rpc_tcp_lock);
        
  /* Allocate new request_p */
  req_p = (struct request_p *) kmalloc(sizeof(struct request_p), GFP_ATOMIC);
  if (!req_p) {
    RPC_PERROR("can't allocate request_p\n");
    return;
  }

  req_p->list.next = NULL;
  req_p->list.prev = NULL;
  req_p->xid = xid;
  req_p->ip = ip;
  req_p->blob = blob;
  req_p->proto = proto;

  /* Initialize timer */
  init_timer(&req_p->timeout);
  req_p->timeout.expires = jiffies + EXP;
  req_p->timeout.data = (unsigned long)req_p;
  req_p->timeout.function = delete_request_p;
  add_timer(&req_p->timeout); 

  /* Put into list */
  WRITE_LOCK(&ipct_rpc_tcp_lock);
  list_prepend(&request_p_list_tcp, req_p);
  WRITE_UNLOCK(&ipct_rpc_tcp_lock); 
  return; 

}

static int add_expect_related(u_int32_t protocol,
                              u_int32_t port,
                              int dir,
                              struct ip_conntrack *ct,
			      int* verdict)
{
  struct ip_conntrack_expect *exp;
  int tmp_ret;

  *verdict = NF_ACCEPT;

  /* initialize struct expect */
  exp = ip_conntrack_expect_alloc();
  if (!exp) {
    RPC_PERROR("cannot allocate space for a conntrack expectation");
    *verdict = NF_DROP;
    return 1; /* this is an error situation */
  }

  exp->tuple.src.ip = ct->tuplehash[!dir].tuple.src.ip;
  exp->tuple.dst.ip = ct->tuplehash[!dir].tuple.dst.ip;
  exp->mask.src.ip = 0xffffffff;
  exp->mask.dst.ip = 0xffffffff;

  switch (protocol) {
  case IPPROTO_UDP:
    exp->tuple.src.u.udp.port = 0;
    exp->tuple.dst.u.udp.port = htons(port);
    exp->tuple.dst.protonum = IPPROTO_UDP;
    exp->mask.src.u.udp.port = 0;
    exp->mask.dst.u.udp.port = htons(0xffff);
    exp->mask.dst.protonum = 0xff;
    break;
      
  default :
    exp->tuple.src.u.tcp.port = 0;
    exp->tuple.dst.u.tcp.port = htons(port);
    exp->tuple.dst.protonum = IPPROTO_TCP;
    exp->mask.src.u.tcp.port = 0;
    exp->mask.dst.u.tcp.port = htons(0xffff);
    exp->mask.dst.protonum = 0xff;
    break;
  }

  exp->expectfn = NULL;
  exp->master = ct;

  if (exp->master->helper == NULL) {
    RPC_PWARN("master helper NULL");
    *verdict = NF_ACCEPT;
    return 0; /* we do not rate this as an error */
  }

  tmp_ret = ip_conntrack_expect_related(exp);

  if (tmp_ret) {
    /* this is not an error */
    RPC_PDEBUG_L1("cannot est. expect related ip %u.%u.%u.%u:0-%u.%u.%u.%u:%u proto=%u: 0x%x\n",
		  NIPQUAD(exp->tuple.src.ip),
		  NIPQUAD(exp->tuple.dst.ip),
		  port, protocol,
		  tmp_ret);
    ip_conntrack_expect_free(exp);

  } else {
    RPC_PDEBUG_L1("expect related ip %u.%u.%u.%u:0-%u.%u.%u.%u:%u proto=%u.\n",
		  NIPQUAD(exp->tuple.src.ip),
		  NIPQUAD(exp->tuple.dst.ip),
		  port, protocol);
  }

  *verdict = NF_ACCEPT;
  return 0;
}

static int check_rpc_packet(const u_int32_t *data,
                            int dir, struct ip_conntrack *ct,
                            struct list_head request_p_list,
                            int rpc_len)
{
  u_int32_t xid;
  int ret = NF_ACCEPT;
  int tmp_ret;

  u_int32_t prog_num;
  u_int32_t proc_num;
  u_int32_t proto_num;
  u_int16_t blob;
  int port_index,port_bad; /* for the search loop */

  struct request_p *req_p;

  /* Translation's buffer for XDR */
  u_int16_t port_buf=0;

  /* Since this is a RPC call/reply via TCP the data is encapsulated into
   * RM fragments. Each RM fragment has a 32bit header that we skip here.
   * See RFC 1050 for details.
   *
   * Due to our sanity checks done before we know that we have at least 28 bytes
   * in the rpc data.
   * A data++ skips 4 of them.  
   */

  /* Skip the RM header */
  data++;

  /* Get XID */
  xid = *data;

  /* Perform direction dependant sanity checking and processing on RPC payloads. */
  if (dir == IP_CT_DIR_ORIGINAL) {
    /* In this direction we track:
     * - RPC call program number portmapper (100000) procedure GETPORT (3)
     *   This is the standard case
     * - RPC call program number ??? (390512) procedure GETPORT (3)
     *   This is some proprietary RPC call to the albd server of ClearCase.
     *   We track the call in order to learn about the server ports that are
     *   communicated to the caller. The tracking is derived from the standard case.
     *
     * We do not check for the RPC call program number portmapper (100000)
     * procedure DUMP (4). This is done by the matcher module (see ipt_rpc.c).
     */

    /* Get the RPC program number used in this call */
    data += 3;
    prog_num = IXDR_GET_INT32(data);

    /* Get the RPC procedure number used in this call */
    data += 2;
    proc_num = IXDR_GET_INT32(data);

    if ( prog_num == 100000 && proc_num == 3 ) {
      RPC_PDEBUG_L1("Detected RPC call to program number\n");
      RPC_PDEBUG_L1("portmapper [%u], procedure GETPORT [%u]. [parse]\n",
		    prog_num, proc_num);

      /* Do an additional length check.
       * Data beyond the verifier are not guaranteed!
       * However we passed the length of the rpc payload.
       */
      if ( rpc_len != 60 )  {
	RPC_PDEBUG_L2("packet length is not correct for a RPC call via TCP. [skip]\n");
	goto dir_call_out;
      }

      /* For the normal portmapper GETPORT we extract the requested protocol
       * as well
       * Advance pointer to the Credentials */
      data++;

      /* Jump Credentials and Verfifier */
      data += 2;
      data += 2;

      /* Print (do not store) RPC program number inside the GETPORT query */
      RPC_PDEBUG_L2("RPC call contains a query for program [%u]. [cont]\n",
		    (unsigned int)IXDR_GET_INT32(data));

      /* Get and store RPC protocol number inside the GETPORT query */
      data += 2;
      proto_num = IXDR_GET_INT32(data);

      /* set kind mask for the blob data */
      blob = REQUEST_P_BLOB_KIND_GETPORT;

    } else if ( prog_num == 390512 && proc_num == 3 ) {
      RPC_PDEBUG_L1("Detected proprietary ClearCase RPC call to\n");
      RPC_PDEBUG_L1("program number [%u], procedure GETPORT [%u]. [parse]\n",
		    prog_num, proc_num);
      /* For the proprietary ClearCase GETPORT call we face then following problem:
       * We (currently) lack the knowledge howto derive the protocol that the expected call
       * will use from the GETPORT call.
       * We saw both UDP and TCP calls follow a UDP GETPORT.
       * We therefore add expectations for both UDP and TCP on receipt of the RPC reply!
       * However, in the alloc_request_p_tcp we just store the TCP part.
       */
      proto_num = (unsigned int) IPPROTO_TCP;
      /* set kind mask for the blob data */
      blob = REQUEST_P_BLOB_KIND_GETPORT_ALBD;

    } else {
      RPC_PDEBUG_L2("RPC call to untracked program number [%u],procedure [%u]. [skip]\n",
		    prog_num, proc_num);
      goto dir_call_out;
    }

    /*
     * Store the data against the RPC clients IP  parameters
     * Note: the u_int32_t proto_num is casted to the u_int16_t proto parameter
     * of function alloc_request_p_tcp.
     */

    alloc_request_p_tcp(xid, ct->tuplehash[dir].tuple.src.ip,
                        blob,(u_int16_t) proto_num );
        
    RPC_PDEBUG_L2("allocated RPC req_p for xid=%x proto=%u ip=%u.%u.%u.%u [req]\n",
		  xid, proto_num,
		  NIPQUAD(ct->tuplehash[dir].tuple.src.ip));

  dir_call_out:
    return ret;

  } else {
    /*
     * In this direction we track:
     * - RPC reply for program number portmapper (100000) procedure GETPORT (3)
     *   This is the standard case
     * - RPC reply for program number ??? (390512) procedure ??? (3)
     *   This is some proprietary RPC reply from the albd server of ClearCase.
     *   We track the call in order to learn about the server ports that are
     *   communicated to the caller. The tracking is derived from the standard case.
     *
     * We do not check for the RPC reply for the program number portmapper (100000)
     * procedure DUMP (4). This is done in the matcher module (see ipt_rpc.c)
     */

    /* Check for returning packet's stored counterpart
     *
     * By using the equality predicate request_p_cmp_kind_getport we select
     * only the stored requests of kind
     *     REQUEST_P_BLOB_KIND_GETPORT
     * and REQUEST_P_BLOB_KIND_GETPORT_ALBD
     *
     * We intentionaly ignore here stored requests of a different kind.
     * Note: currently there are none of other kind.
     */
    WRITE_LOCK(&ipct_rpc_tcp_lock);
    req_p = LIST_FIND(&request_p_list_tcp, request_p_cmp_kind_getport,
		      struct request_p *, xid,
		      ct->tuplehash[!dir].tuple.src.ip);
    WRITE_UNLOCK(&ipct_rpc_tcp_lock);

    /* Skip unexpected packets. Note: this includes replies to DUMP calls.
     * They are of a different kind and will not be found by
     * the compare function used above.
     */
    if (!req_p) {
      RPC_PDEBUG_L2("RPC reply packet is not expected by helper. [skip]\n");
      goto dir_reply_out;
    }

    /* We found a request for that reply packet. 
     * The reply packet is either of kind
     *    REQUEST_P_BLOB_KIND_GETPORT
     * or REQUEST_P_BLOB_KIND_GETPORT_ALBD
     */

    /* Advance to the field MsgType of the RPC header   */
    data++;
    /* Verify if packet is really an RPC reply packet */
    if (IXDR_GET_INT32(data) != 1) {
      RPC_PDEBUG_L2("The MsgType of the packet is not 'reply'. [skip]\n");
      goto dir_reply_out_clear_req;
    }

    /* Advance to the field ReplyState of the RPC body  */
    data++;
    /* Is status accept? */
    if (IXDR_GET_INT32(data)) {
      RPC_PDEBUG_L2("The ReplyState of the packet is not 'accept'. [skip]\n");
      goto dir_reply_out_clear_req;
    }

    /* Advance to the field Verifier of the RPC body  */
    data++;
    /* Advance to the field AcceptStatus of the RPC body  */
    data += 2;
    /* Is accept status "success"? */
    if (IXDR_GET_INT32(data)) {
      RPC_PDEBUG_L2("The AcceptStatus of the packet is not 'success'. [skip]\n");
      goto dir_reply_out_clear_req;
    }

    /* Depending on the REQUEST_P_BLOB_KIND we do the further processing */
    switch ((req_p->blob) & REQUEST_P_BLOB_KIND_MASK) {
    case REQUEST_P_BLOB_KIND_GETPORT:
      /* For the standard GETPORT reply we extract the server port number 
       * Advance to the next data field of the RPC body  
       * It contains the server port. 
       */
      if ( rpc_len == 32 ) {
	/* the rpc data contains the correct number of bytes for a
	 * TCP GETPORT reply
	 * 28 (mandatory prefix) + 4 bytes payload
	 */
	data++;
	port_buf = (u_int16_t) IXDR_GET_INT32(data);
      } else {
	RPC_PDEBUG_L2("packet len %d is incorrect for GETPORT. [skip]\n",
		      rpc_len);
	port_buf = 0;
      }
      /* If a packet has made it this far then it deserves an expectation.
       * However, if port == 0, then this service is not going to be
       * registered as expected.
       * We also check that we do not add expectations for ports that we
       * supervise via the helper. use the global array ports[MAX_PORTS]
       */

      /* Check if we would add an expectation for the port of some portmapper that we supervise */
      for (port_index=0,port_bad=0; port_index < ports_n_c; port_index++) {
	if (port_buf == ports[port_index]) {
	  port_bad++;
	  RPC_PINFO("skipping addition of expectation for supervised portmapper port %u\n",
		    port_buf);         
	  break;
	}
      }

      if (port_buf && port_buf != nsrexec && !port_bad) {
	RPC_PDEBUG_L1("port found: %u\n", port_buf);
	tmp_ret = add_expect_related(req_p->proto,port_buf,dir,ct,&ret);
      }
      break;
    case REQUEST_P_BLOB_KIND_GETPORT_ALBD:
      /* For the ClearCase ALBD GETPORT reply we extract the server port
       * number.
       * Advance 10 data fields in the RPC body.  
       * This sets the pointer to the field containing the server port.
       */
      if ( rpc_len == 68 ) {
	/* the rpc data contains the correct number of bytes for a
	 * TCP ALBD GETPORT reply which are 28 (mandatory prefix)
	 * + 40 bytes payload.
	 * We have to advance the pointer by 1 to get to the payload
	 * and then 9 to get to the server port
	 */
	data += 10;
	port_buf = (u_int16_t) IXDR_GET_INT32(data);
      } else {
	RPC_PDEBUG_L2("packet len %d is incorrect for ALBD GETPORT. [skip]\n",
		      rpc_len);
	port_buf = 0;
      }
      /* If a packet has made it this far then it deserves an expectation.
       * However, if port == 0, then this service is not going to be
       * registered as expected.
       *
       * Problem: the related connection to the CC GETPORT is either via TCP or UDP
       *          We do not know how the predict this from the RPC call
       * Solution: we add an expect_relate for UDP and TCP
       *           Since we do not check for authorized programs for the CC traffic
       *           in the matcher module this is not a problem.
       * We also check that we do not add expectations for ports that we
       * supervise via the helper. use the global array ports[MAX_PORTS]
       */

      /* Check if we would add an expectation for the port of some portmapper that we supervise */
      for (port_index=0,port_bad=0; port_index < ports_n_c; port_index++) {
	if (port_buf == ports[port_index]) {
	  port_bad++;
	  RPC_PINFO("skipping addition of expectation for supervised portmapper port %u\n",
		    port_buf);         
	  break;
	}
      }

      if (port_buf && !port_bad) {
	RPC_PDEBUG_L1("port found: %u\n", port_buf);

	/* add the expectation for a UDP connection */
	tmp_ret = add_expect_related(IPPROTO_UDP,port_buf,dir,ct,&ret);
	/* add the expectation for a TCP connection */
	tmp_ret = add_expect_related(IPPROTO_TCP,port_buf,dir,ct,&ret);
      }
      break;
    } /* switch ((req_p->blob) & REQUEST_P_BLOB_KIND_MASK) */

  dir_reply_out_clear_req:
    /* Remove the stored request from the list. We no longer need it */
    req_cl(req_p);

  dir_reply_out:
    RPC_PDEBUG_L2("RPC reply packet evaluated. [expect]\n");
    return ret;
  } /* DIRECTION REPLY*/

    /* Some kind of catch all; should never be reached */
  return ret;

}


/* RPC TCP helper */
static int help(struct sk_buff **pskb,
		struct ip_conntrack *ct, enum ip_conntrack_info ctinfo)
{
  int dir;
  int crp_ret;
  struct tcphdr _tcph, *tcph;
  const u_int32_t *data;
  size_t tcplen;
  struct iphdr *iph;
  size_t len;

  if (ct == NULL) {
    RPC_PWARN("ct is NULL");
    return NF_ACCEPT;
  }

  /* Not whole TCP header? Check 1 */
  iph=(*pskb)->nh.iph;
  tcph = skb_header_pointer(*pskb,iph->ihl*4,sizeof(_tcph),&_tcph);
  if (!tcph)
    return NF_ACCEPT;

  len = (*pskb)->len; /* stes */
  data = (const u_int32_t *)tcph + tcph->doff;
  tcplen = len - iph->ihl * 4;
  dir = CTINFO2DIR(ctinfo);

  RPC_PDEBUG_L2("new packet to evaluate ...\n");

  /* This works for packets like handshake packets, ignore */
  if (len == ((tcph->doff + iph->ihl) * 4)) {
    RPC_PDEBUG_L2("packet has no data (may still be handshaking). [skip]\n");
    return NF_ACCEPT;
  }

  /* Until there's been traffic both ways, don't look into packets. */
  if (ctinfo != IP_CT_ESTABLISHED
      && ctinfo != IP_CT_ESTABLISHED + IP_CT_IS_REPLY) {
    RPC_PWARN("connection tracking state is; ctinfo=%u ..\n", ctinfo);
    RPC_PWARN("[note: failure to get past this error\n");
    RPC_PWARN("[      may indicate asymmetric routing]\n");
    RPC_PWARN("packet is not yet part of a two way stream. [skip]\n");
    return NF_ACCEPT;
  }

  /* Not whole TCP header? Check 2 */
  if (tcplen < sizeof(struct tcphdr) || tcplen < tcph->doff * 4) {
    RPC_PWARN("TCP header length is; tcplen=%u ..\n", (unsigned) tcplen);
    RPC_PWARN("packet does not contain a complete TCP header. [skip]\n");
    return NF_ACCEPT;
  }

  /* FIXME: Source route IP option packets --RR */
  if (tcp_v4_check(tcph, tcplen, iph->saddr, iph->daddr,
		   csum_partial((char *) tcph, tcplen, 0))) {
    RPC_PWARN("csum; %p %u %u.%u.%u.%u %u.%u.%u.%u\n",
	      tcph, tcplen, NIPQUAD(iph->saddr),
	      NIPQUAD(iph->daddr));
    RPC_PWARN("[note: failure to get past this error may indicate source routing]\n");
    RPC_PWARN("packet contains a bad checksum. [skip]\n");
    return NF_ACCEPT;
  }

  /* Perform direction dependant protocol work
   * Since we are interested in more than just the simple portmapper/GETPORT call
   * we changed the sanity checks at this place. 
   */

  if (dir == IP_CT_DIR_ORIGINAL) {
    RPC_PDEBUG_L2("packet is from the initiator. [cont]\n");
    /* Tests if packet len is plausible
     *  
     *  A RPC call via TCP (initial RM frag) consists at least of the following data:
     *  04 bytes RM header
     *  08 bytes RPC call header
     *         04 bytes XID
     *         04 bytes MsgType
     *  32 bytes RPC call body (mandatory prefix)
     *         04 bytes RpcVerion
     *         04 bytes RpcProgramNumber
     *         04 bytes RpcProgramVersion
     *         04 bytes RpcProcedureNumber
     *        >=8 bytes Credentials: length is variable
     *         08 bytes Verifier
     *
     *  In sum this is at least 44 bytes
     *
     *  The rest depends on the program and procedure number. E.g.
     *  - portmapper (100000)/GETPORT (3): 16 bytes
     *             04 bytes QueryRpcProgram
     *             04 bytes QueryRpcVersion
     *             04 bytes QueryProtocol
     *             04 bytes QueryPort
     *  - portmapper (100000)/DUMP    (4):  0 bytes
     *  - albd       (390512)/GETPORT (3): length depends on credentials
     *
     */

    /* 
     * Check for exact length of the call.
     * Otherwise bail out. We are not interested in DUMP calls
     */
    if ( (tcplen - (tcph->doff * 4)) < 44 ) {
      RPC_PDEBUG_L2("packet length is not correct for a RPC call via TCP. [skip]\n");
      return NF_ACCEPT;
    }

  } else {
    RPC_PDEBUG_L2("packet is from the receiver. [cont]\n");
    /* Tests if packet len is plausible
     *
     *  A RPC reply via TCP (initial RM frag) consists at least of the following data:
     *  04 bytes RM header
     *  08 bytes RPC reply header
     *       04 bytes XID
     *       04 bytes MsgType
     *  16 bytes RPC reply body (mandatory prefix)
     *       04 bytes ReplyState
     *       08 bytes Verifier
     *       04 bytes AcceptState
     *
     *  In sum this is 28 bytes
     *
     *  The rest depends on the program and procedure number. E.g.
     *  - portmapper (100000)/GETPORT (3):  4 bytes
     *             04 bytes AnswerPort
     *  - portmapper (100000)/DUMP    (4): arbitrary (depends on the number of
     *                                     registered programs)
     *  - albd       (390512)/GETPORT (3): 40 bytes
     */

    /* 
     * Check for exact length of the reply
     * Otherwise bail out. We are not interested in DUMP replies
     */
    if ( ((tcplen - (tcph->doff * 4)) != 32 ) &&  /* 28 + 4  */
	 ((tcplen - (tcph->doff * 4)) != 68 )) {  /* 28 + 40 */
      RPC_PDEBUG_L2("packet length is not correct for a RCP GETPORT reply via TCP. [skip]\n");
      return NF_ACCEPT;
    }
  }

  /* Check the RPC data */
  crp_ret = check_rpc_packet(data, dir, ct, request_p_list_tcp,
			     tcplen - (tcph->doff * 4));

  return crp_ret;

}

static struct ip_conntrack_helper rpc_helpers[MAX_PORTS];
static char rpc_names[MAX_PORTS][10];

static void fini(void);

static int __init init(void)
{
  int port, ret;
  char *tmpname;

  /* If no port given, default to standard RPC port */
  if (ports[0] == 0)
    ports[0] = RPC_PORT;

  for (port = 0; (port < MAX_PORTS) && ports[port]; port++) {
    memset(&rpc_helpers[port], 0, sizeof(struct ip_conntrack_helper));

    tmpname = &rpc_names[port][0];
    if (ports[port] == RPC_PORT)
      sprintf(tmpname, "rpc");
    else
      sprintf(tmpname, "rpc-%d", ports[port]);
    rpc_helpers[port].name = tmpname;

    rpc_helpers[port].me = THIS_MODULE;
    rpc_helpers[port].max_expected = RPC_CONNTRACK_DUMP_MAX_EXPECTED;
    /* FABR: we had to raise max_expected from 1 to RPC_CONNTRACK_DUMP_MAX_EXPECTED
     * in order to allow for multiple expectations caused by a DUMP reply.
     * The DUMP reply contains the server ports.
     * A matching rule in the chain for the inbound traffic from the RPC server like e.g.
     *  ${IPTABLES} -A forward_cells -s 192.168.3.2 -p tcp --source-port 111 \
     *              -m rpc --rpcs 100003,100005 -j ACCEPT
     * specifies which one of these are allowed. In the example we allow only for 2 programs.
     *
     * We need to be able to register as many simultaneous ct expectations as we maximally allow in
     * any matching rule that matches DUMP replies.
     * Technically, the limit would be IPT_RPC_MAX_ENTS (see iptables.../extensions/libipt_rpc.c)
     * However, we decided to use some smaller value (RPC_CONNTRACK_DUMP_MAX_EXPECTED).
     * Anyways, the value IPT_RPC_MAX_ENTS isn't defined in any header file we like to include.
     *
     */
    rpc_helpers[port].timeout = 5 * 60; /* stes */

    rpc_helpers[port].tuple.dst.protonum = IPPROTO_TCP;
    rpc_helpers[port].mask.dst.protonum = 0xff;

    /* RPC can come from ports 0:65535 to ports[port] (111) */
    rpc_helpers[port].tuple.src.u.tcp.port = htons(ports[port]);
    rpc_helpers[port].mask.src.u.tcp.port = htons(0xffff);
    rpc_helpers[port].mask.dst.u.tcp.port = htons(0x0);

    rpc_helpers[port].help = help;

    RPC_PINFO("registering helper for port #%d: %d/TCP\n", port, ports[port]);
    RPC_PINFO("helper match ip   %u.%u.%u.%u:%u->%u.%u.%u.%u:%u\n",
	      NIPQUAD(rpc_helpers[port].tuple.dst.ip),
	      ntohs(rpc_helpers[port].tuple.dst.u.tcp.port),
	      NIPQUAD(rpc_helpers[port].tuple.src.ip),
	      ntohs(rpc_helpers[port].tuple.src.u.tcp.port));
    RPC_PINFO("helper match mask %u.%u.%u.%u:%u->%u.%u.%u.%u:%u\n",
	      NIPQUAD(rpc_helpers[port].mask.dst.ip),
	      ntohs(rpc_helpers[port].mask.dst.u.tcp.port),
	      NIPQUAD(rpc_helpers[port].mask.src.ip),
	      ntohs(rpc_helpers[port].mask.src.u.tcp.port));

    ret = ip_conntrack_helper_register(&rpc_helpers[port]);

    if (ret) {
      printk("ERROR registering port %d\n",
	     ports[port]);
      fini();
      return -EBUSY;
    }
    ports_n_c++;
  }
  RPC_PINFO("%s Legato NetWorker support for port %d/TCP\n", (nsrexec)?"enabling":"disabling", nsrexec);

  return 0;
}


/* This function is intentionally _NOT_ defined as __exit, because 
 * it is needed by the init function */
static void fini(void)
{
  int port;

  RPC_PINFO("cleaning request list\n");
  clean_request(&request_p_list_tcp);

  for (port = 0; (port < ports_n_c) && ports[port]; port++) {
    RPC_PINFO("unregistering port %d\n", ports[port]);
    ip_conntrack_helper_unregister(&rpc_helpers[port]);
  }
}


module_init(init);
module_exit(fini);

struct module *ip_conntrack_rpc_tcp = THIS_MODULE;
EXPORT_SYMBOL(request_p_list_tcp);
EXPORT_SYMBOL(ip_conntrack_rpc_tcp);
EXPORT_SYMBOL(ipct_rpc_tcp_lock);
