/* RPC extension for IP connection matching, Version 2.2
 * (C) 2000 by Marcelo Barbosa Lima <marcelo.lima@dcc.unicamp.br>
 *      - original rpc tracking module
 *      - "recent" connection handling for kernel 2.3+ netfilter
 *
 * (C) 2001 by Rusty Russell <rusty@rustcorp.com.au>
 *      - upgraded conntrack modules to oldnat api - kernel 2.4.0+
 *
 * (C) 2002,2003 by Ian (Larry) Latter <Ian.Latter@mq.edu.au>
 *      - upgraded conntrack modules to newnat api - kernel 2.4.20+
 *      - extended matching to support filtering on procedures
 *
 * (c) 2007,2008 by Franz Regensburger (FABR) <franz.regensburger@odorf.de>
 *      Removed bug with respect to usage of LIST_FIND in check_rpc_packet.
 *      Do not pass the structure 'struct list_head request_p_list' as parameter.
 *      You must use the global variables request_p_list_tcp, request_p_list_upd
 *      in macro calls
 *         LIST_FIND(&request_p_list_tcp, ...
 *         LIST_FIND(&request_p_list_upd, ...
 *      Otherwise the macros will not work properly! They need the original head pointer.
 *       
 * (c) 2007,2008 by Franz Regensburger (FABR) <franz.regensburger@odorf.de>
 *      Added tracking support for
 *        - rpc program 100000 (portmapper) procedure DUMP
 *        - proprietary RPC based negotiation of server ports for Clear Case
 *          (albd daemon)
 *
 *      Note: support for rpc replies to rpc call program 100000 (portmapper) procedure DUMP
 *      is important if there are some Linux clients that do nfs via your firewall.
 *      Linux nfs clients always use a DUMP call to the portmapper of the nfs server
 *      in order to learn about available RPC programs. They use the DUMP reply to decide
 *      which mount RPC call to launch. They never do a GETPORT for the mount program [100005].
 *      However, they launch a GETPORT for the nfs program [100003].
 *      As a result linux nfs clients need the DUMP support in the rpc connection tracker.
 *
 *
 * ipt_rpc.c,v 2.2 2003/01/12 18:30:00
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License
 *      as published by the Free Software Foundation; either version
 *      2 of the License, or (at your option) any later version.
 **
 *      Module load syntax:
 *      insmod ipt_rpc.o ports=port1,port2,...port<MAX_PORTS>
 *
 *      Please give the ports of all RPC servers you wish to connect to.
 *      For example, ports=111,371,7938 where
 *         111 is the standard portmapper port
 *         371 is for the ClearCase ALBD portmapper 
 *        7938 is for Legato NetWorker's portmapper
 *
 *      If you don't specify ports, the default will be port 111 (SUN portmap).
 **
 *      Note to all:
 *
 *      RPCs should not be exposed to the internet - ask the Pentagon;
 *
 *        "The unidentified crackers pleaded guilty in July to charges
 *         of juvenile delinquency stemming from a string of Pentagon
 *         network intrusions in February.
 *
 *         The youths, going by the names TooShort and Makaveli, used
 *         a common server security hole to break in, according to
 *         Dane Jasper, owner of the California Internet service
 *         provider, Sonic. They used the hole, known as the 'statd'
 *         exploit, to attempt more than 800 break-ins, Jasper said."
 *
 *      From: Wired News; "Pentagon Kids Kicked Off Grid" - Nov 6, 1998
 *      URL:  http://www.wired.com/news/politics/0,1283,16098,00.html
 **
 */

#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/list.h>
#include <linux/udp.h>
#include <linux/tcp.h>
#include <net/tcp.h>
#include <linux/netfilter_ipv4/ip_conntrack.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ip_conntrack_rpc.h>
#include <linux/netfilter_ipv4/lockhelp.h>
#include <linux/netfilter_ipv4/ipt_rpc.h>

#define MAX_PORTS 8
static int ports[MAX_PORTS];
static int ports_n_c = 0;

#ifdef MODULE_PARM
MODULE_PARM(ports, "1-" __MODULE_STRING(MAX_PORTS) "i");
MODULE_PARM_DESC(ports, "port numbers (TCP/UDP) of RPC portmapper servers");
#endif

MODULE_AUTHOR("Marcelo Barbosa Lima <marcelo.lima@dcc.unicamp.br>");
MODULE_DESCRIPTION("RPC connection matching module");
MODULE_LICENSE("GPL");

/* Here we define the amount of output we like.
 * Use define or undef according to your preferences.
 */

#define RPC_DO_PINFO
#define RPC_DO_PERROR
#define RPC_DO_PWARN
#undef  RPC_DO_PDEBUG_L1
#undef  RPC_DO_PDEBUG_L2
#undef  RPC_DO_PDEBUG_L3

/* And here we define the output functions depending on the above settings */ 
#define RPC_PROGNAME "ipt_rpc: "
#ifdef  RPC_DO_PINFO
#define RPC_PINFO(format, args...) printk(KERN_DEBUG RPC_PROGNAME format, ## args)
#else
#define RPC_PINFO(format, args...)
#endif

#ifdef  RPC_DO_PERROR
#define RPC_PERROR(format, args...) printk(KERN_DEBUG RPC_PROGNAME format, ## args)
#else
#define RPC_PERROR(format, args...)
#endif

#ifdef  RPC_DO_PWARN
#define RPC_PWARN(format, args...) printk(KERN_DEBUG RPC_PROGNAME format, ## args)
#else
#define RPC_PWARN(format, args...)
#endif

#ifdef  RPC_DO_PDEBUG_L1
#define RPC_PDEBUG_L1(format, args...) printk(KERN_DEBUG RPC_PROGNAME format, ## args)
#else
#define RPC_PDEBUG_L1(format, args...)
#endif

#ifdef  RPC_DO_PDEBUG_L2
#define RPC_PDEBUG_L2(format, args...) printk(KERN_DEBUG RPC_PROGNAME format, ## args)
#else
#define RPC_PDEBUG_L2(format, args...)
#endif

#ifdef  RPC_DO_PDEBUG_L3
#define RPC_PDEBUG_L3(format, args...) printk(KERN_DEBUG RPC_PROGNAME format, ## args)
#else
#define RPC_PDEBUG_L3(format, args...)
#endif

EXPORT_NO_SYMBOLS;

/* our own lock for tracking RM fragments in a TCP stream */
static DECLARE_LOCK(ipct_rpc_lock);

/* externals from ip_conntrack_rpc_tcp */
extern struct list_head request_p_list_tcp;
extern struct module *ip_conntrack_rpc_tcp;

/* externals from ip_conntrack_rpc_udp */
extern struct list_head request_p_list_udp;
extern struct module *ip_conntrack_rpc_udp;

DECLARE_RWLOCK_EXTERN(ipct_rpc_tcp_lock);
DECLARE_RWLOCK_EXTERN(ipct_rpc_udp_lock);

#define ASSERT_READ_LOCK(x)                                     \
do {                                                            \
        if (x == &request_p_list_udp)                           \
                MUST_BE_READ_LOCKED(&ipct_rpc_udp_lock);        \
        else if (x == &request_p_list_tcp)                      \
                MUST_BE_READ_LOCKED(&ipct_rpc_tcp_lock);        \
} while (0)

#define ASSERT_WRITE_LOCK(x)                                    \
do {                                                            \
        if (x == &request_p_list_udp)                           \
                MUST_BE_WRITE_LOCKED(&ipct_rpc_udp_lock);       \
        else if (x == &request_p_list_tcp)                      \
                MUST_BE_WRITE_LOCKED(&ipct_rpc_tcp_lock);       \
} while (0)

#include <linux/netfilter_ipv4/listhelp.h>

const int IPT_RPC_CHAR_LEN = 11;


static int k_atoi(char *string)
{
  unsigned int result = 0;
  int maxoctet = IPT_RPC_CHAR_LEN;

  for ( ; *string != 0 && maxoctet != 0; maxoctet--, string++) {
    if (*string < 0)
      return(0);
    if (*string == 0)
      break;
    if (*string < 48 || *string > 57) {
      return(0);
    }
    result = result * 10 + ( *string - 48 );
  }
  return(result);
}


static int match_rpcs(char *c_procs, int i_procs, int proc)
{
  /* This function matches the RPC program number used in the RPC GETPORT call
   * or those listed in the RPC DUMP reply against the list of allowed program numbers
   * we extracted from the firewall rule.
   * Returns == 0 : the program number is not in the list of allowed programs
   * Returns == 1 : the program number is in the list of allowed programs or the
   *                list is empty (all are allowed)
   */
  int   proc_ctr;
  char *proc_ptr;
  unsigned int proc_num;

  if (i_procs == -1)
    return 1;

  for (proc_ctr=0; proc_ctr <= i_procs; proc_ctr++) {

    proc_ptr = c_procs;
    proc_ptr += proc_ctr * IPT_RPC_CHAR_LEN;
    proc_num = k_atoi(proc_ptr);

    if (proc_num == proc)
      return 1;
  }

  return 0;
}


static void add_expect_related(u_int32_t carrier_protocol,
			       u_int32_t map_protocol,
			       u_int32_t map_port,
			       int dir,
			       struct ip_conntrack *ct,
			       u_int32_t match_seqnum)
{
  struct ip_conntrack_expect expect;
  struct ip_conntrack_expect *exp;
  int ret;
  int port_index,port_bad; /* for the search loop */

  exp = &expect;

  /* Note:
   * map_port is the one we read via IXDR_GET_INT32(data)
   * Therefore, it is in host order!
   *
   * We also check that we do not add expectations for ports that we
   * supervise via the helper. use the global array ports[MAX_PORTS]
   */

  /* Check if we would add an expectation for the port of some portmapper that we supervise */
  for (port_index=0,port_bad=0; port_index < ports_n_c; port_index++) {
    if (map_port == ports[port_index]) {
      port_bad++;
      RPC_PINFO("skipping addition of expectation for supervised portmapper port %u\n",
		map_port);         
      return;
    }
  }

  /* initialize struct expect */
  memset(&expect, 0, sizeof(expect));

  exp->tuple.src.ip = ct->tuplehash[!dir].tuple.src.ip;
  exp->tuple.dst.ip = ct->tuplehash[!dir].tuple.dst.ip;
  exp->mask.src.ip = 0xffffffff;
  exp->mask.dst.ip = 0xffffffff;

  switch (map_protocol) {
  case IPPROTO_UDP:
    exp->tuple.src.u.udp.port = 0;
    exp->tuple.dst.u.udp.port = htons(map_port);
    exp->tuple.dst.protonum = IPPROTO_UDP;
    exp->mask.src.u.udp.port = 0;
    exp->mask.dst.u.udp.port = htons(0xffff);
    exp->mask.dst.protonum = 0xffff;
    break;
            
  default :
    /* Setting seq is only sensible if we track a TCP stream */
    if (carrier_protocol == IPPROTO_TCP) {
      exp->seq=match_seqnum;
    }
    exp->tuple.src.u.tcp.port = 0;
    exp->tuple.dst.u.tcp.port = htons(map_port);
    exp->tuple.dst.protonum = IPPROTO_TCP;
    exp->mask.src.u.tcp.port = 0;
    exp->mask.dst.u.tcp.port = htons(0xffff);
    exp->mask.dst.protonum = 0xffff;
    break;
  }
  exp->expectfn = NULL;
    
  ret= ip_conntrack_expect_related(ct, &expect);

  if (ret) {
    RPC_PDEBUG_L1("cannot est. expect related ip %u.%u.%u.%u:0-%u.%u.%u.%u:%u proto=%u: 0x%x\n",
		  NIPQUAD(exp->tuple.src.ip),
		  NIPQUAD(exp->tuple.dst.ip),
		  map_port, map_protocol,
		  ret);
  } else {
    RPC_PDEBUG_L1("expect related ip %u.%u.%u.%u:0-%u.%u.%u.%u:%u proto=%u.\n",
		  NIPQUAD(exp->tuple.src.ip),
		  NIPQUAD(exp->tuple.dst.ip),
		  map_port, map_protocol);
  }

  return;
}

static int scan_map_list(const u_int32_t *data,
                         const void *matchinfo,
                         int dir,
                         struct ip_conntrack *ct,
                         u_int32_t carrier_protocol,
                         u_int16_t maplen,
                         u_int32_t new_seqnum,
                         u_int16_t old_flags, 
                         u_int16_t* new_flags 
                         )
{
  /* This function scans the map list contained in the DUMP reply packets
   * For UDP there is only on such packet.
   * FOR TCP there is a series of RM fragments that contain the map list.
   * Hence, for TCP we face the challenge to keep track of the fragmented map entries that
   * are split over two consecutive RM fragments.
   */
  const struct ipt_rpc_info *rpcinfo = matchinfo;
  u_int16_t mapfate_flag; /* stores the verdict of function match_rpcs encoded into a flag */
  u_int16_t proto_flag; /* stores the protocol of the map entry encoded into a flag */
  u_int16_t missing_bytes; /* stores number of missing bytes read from flags */
  u_int16_t count;
  u_int32_t map_program, map_version, map_protocol, map_port;

  RPC_PDEBUG_L3("scan_map_list: got 0x%x bytes to process\n",maplen);

  count=maplen;

  if ( ! (old_flags & RPC_CT_FLAGS_INITIAL_RM_YES) ) {
    RPC_PDEBUG_L3("start scanning the leading fragmented map entry of a subsequent RM fragment\n");
    missing_bytes = (RPC_CT_FLAGS_MISSING_BYTES_MASK & old_flags);
    RPC_PDEBUG_L3("missing bytes=%d\n",missing_bytes);

    /* check data integrity */
    if ( count < missing_bytes + 4 ) {
      RPC_PERROR("subsequentRM fragment contains only %d bytes\n",count);
      RPC_PERROR("we expect at least %d bytes\n",missing_bytes + 4);
      /* invalidate */
      *new_flags = ( *new_flags & ~(RPC_CT_FLAGS_VALID_YES));
      /* Exit the function here */
      return RPC_PKG_NOMATCH;
    }

    switch (missing_bytes) {
    case 0:
      /* nothing to do */
      break;
    case 4:
      /* 12 bytes seen
       * we have: program, version, protocol
       * we miss: port
       *
       * get FLAGS_MAPFATE
       * get FLAGS_PROTO
       * read port from data
       */
      map_port=IXDR_GET_INT32(data);data++;
      RPC_PDEBUG_L3("map_port=0x%x\n",map_port);

      /* get fate of map entry from the flags */
      mapfate_flag = old_flags & RPC_CT_FLAGS_MAPFATE_MASK;

      /* get protocol from the flags */
      proto_flag = old_flags & RPC_CT_FLAGS_PROTO_MASK;
      if (proto_flag == RPC_CT_FLAGS_PROTO_UDP) {
	map_protocol = IPPROTO_UDP;
      } else {
	/* must be RPC_CT_FLAGS_PROTO_TCP: we support only UDP and TCP yet */
	map_protocol = IPPROTO_TCP;
      }

      if (mapfate_flag == RPC_CT_FLAGS_MAPFATE_PASS) {
	/* program is authorized */
	RPC_PDEBUG_L1("partial map entry belongs to an authorized program request\n");
	add_expect_related(carrier_protocol,map_protocol,map_port,dir,ct,new_seqnum - count);
      } else {
	RPC_PDEBUG_L1("partial map entry belongs to an illegal program request. [drop]\n");
      }
      break;
    case 8:
      /* 8 bytes seen
       * we have: program, version
       * we miss: protocol, port
       *
       * get FLAGS_MAPFATE
       * read proto from data
       * read port from data
       */
      map_protocol=IXDR_GET_INT32(data); data++;
      RPC_PDEBUG_L3("map_protocol=0x%x\n",map_protocol);

      map_port=IXDR_GET_INT32(data);data++;
      RPC_PDEBUG_L3("map_port=0x%x\n",map_port);

      /* get fate of map entry from the flags */
      mapfate_flag = old_flags & RPC_CT_FLAGS_MAPFATE_MASK;

      if (mapfate_flag == RPC_CT_FLAGS_MAPFATE_PASS) {
	/* program is authorized */
	RPC_PDEBUG_L1("partial map entry belongs to an authorized program request\n");
	add_expect_related(carrier_protocol,map_protocol,map_port,dir,ct,new_seqnum - count);
      } else {
	RPC_PDEBUG_L1("partial map entry belongs to an illegal program request. [drop]\n");
      }
      break;
    case 12:
      /* 4 bytes seen
       * we have: program
       * we miss: version, protocol, port
       *
       * get FLAGS_MAPFATE
       * read version from data
       * read proto from data
       * read port from data
       */
      map_version=IXDR_GET_INT32(data); data++;
      RPC_PDEBUG_L3("map_version=0x%x\n",map_version);

      map_protocol=IXDR_GET_INT32(data); data++;
      RPC_PDEBUG_L3("map_protocol=0x%x\n",map_protocol);

      map_port=IXDR_GET_INT32(data);data++;
      RPC_PDEBUG_L3("map_port=0x%x\n",map_port);

      /* get fate of map entry from the flags */
      mapfate_flag = old_flags & RPC_CT_FLAGS_MAPFATE_MASK;

      if (mapfate_flag == RPC_CT_FLAGS_MAPFATE_PASS) {
	/* program is authorized */
	RPC_PDEBUG_L1("partial map entry belongs to an authorized program request\n");
	add_expect_related(carrier_protocol,map_protocol,map_port,dir,ct,new_seqnum - count);
      } else {
	RPC_PDEBUG_L1("partial map entry belongs to an illegal program request. [drop]\n");
      }
      break;
    case 16:
      /* 0 bytes seen
       * we miss: program, version, protocol, port
       *
       * read program from data
       * read version from data
       * read proto from data
       * read port from data
       */
      map_program=IXDR_GET_INT32(data); data++;
      RPC_PDEBUG_L3("map_program=0x%x\n",map_program);

      map_version=IXDR_GET_INT32(data); data++;
      RPC_PDEBUG_L3("map_version=0x%x\n",map_version);

      map_protocol=IXDR_GET_INT32(data); data++;
      RPC_PDEBUG_L3("map_protocol=0x%x\n",map_protocol);

      map_port=IXDR_GET_INT32(data);data++;
      RPC_PDEBUG_L3("map_port=0x%x\n",map_port);

      /* compute fate of map entry */
      if (match_rpcs((char *)&rpcinfo->c_procs,rpcinfo->i_procs,map_program)) {
	/* program is authorized */
	RPC_PDEBUG_L1("map entry contains authorized program request[%u]\n",
		      (unsigned int)map_program);
	add_expect_related(carrier_protocol,map_protocol,map_port,dir,ct,new_seqnum - count);
      } else {
	RPC_PDEBUG_L1("map entry contains illegal program request [%u]. [drop]\n",
		      (unsigned int)map_program);
      }
      break;
    default:
      /* should not happen */
      RPC_PERROR("RM fragment starts with inconsisten partial map of %d bytes\n",
		 missing_bytes);
      /* invalidate */
      *new_flags = ( *new_flags & ~(RPC_CT_FLAGS_VALID_YES));
      /* Exit the function here */
      return RPC_PKG_NOMATCH;
    }

    /* adjust count */
    count = count - missing_bytes;
    RPC_PDEBUG_L3("finished with scanning the leading fragmented map entry\n");
  } else {
    RPC_PDEBUG_L3("start scanning at the head of a map list\n");
  }

  /* At this point we assume that we have processed any fragmented start of
   * the map list entry.
   * Due to the data definition of RPC list structures according to RFC 1014 XDR
   * there must follow at least 4 bytes ('value follows' or 'end of list')
   */

  /* do a consistency check */
  if (count < 4) {
    RPC_PINFO("map entry starts with only %d bytes; invalidating\n",count);
    /* NOTE
     * we get to here via any RPC replies from a supervised portmapper
     * that is not a 'normal' reply to an ordinary GETPORT or a DUMP.
     * E.g. 'rpcinfo -t 192.168.3.2 100000' produces such a reply since
     * the portmapper replies with some special packet if the query for
     * the portmapper itsself.
     */

    /* invalidate */
    *new_flags = ( *new_flags & ~(RPC_CT_FLAGS_VALID_YES));
    /* Exit the function here */
    return RPC_PKG_NOMATCH;
  }

  /* We need at least 4 bytes to play with */
  while (count >= 4) {
    /* At loop entry we assume that pointer data is positioned to the start of a map entry and
     * counter count is set appropriately. Make sure this property holds for all subsequent
     * loop entries.
     *
     * A map entry starts with a 4 byte union qualifier which is either
     * 0x00000000: list end
     * 0x00000001: value follows
     */

    /* Always process a complete list entry in one cycle */
    if (! IXDR_GET_INT32(data)) {
      /* End of map signaled */

      /* Is this consistent with our data? */
      if (count == 4) {
	/* a valid case */
	RPC_PDEBUG_L3("proper end of map reached\n");
	/* force the end of the loop */
	data++;
	count=0;
	/* Set the flags as follows
	 * RPC_CT_FLAGS_MAPFATE_UNDEF
	 * RPC_CT_FLAGS_PROTO_UNDEF
	 * MISSING_BYTES 0
	 */
	*new_flags = 
	  /* the part we leave untouched */
	  ((RPC_CT_FLAGS_VALID_MASK | RPC_CT_FLAGS_INITIAL_RM_MASK) & *new_flags )
	  |
	  /* the part we like to set */
	  RPC_CT_FLAGS_MAPFATE_UNDEF | RPC_CT_FLAGS_PROTO_UNDEF | 0;
      } else {
	/* an error case */
	RPC_PERROR("inconsistent signaling of end of map reached\n");
	/* invalidate */
	*new_flags = ( *new_flags & ~(RPC_CT_FLAGS_VALID_YES));
	/* Exit the function here */
	return RPC_PKG_NOMATCH;
      }
    } else {
      /* value follows; advance 4 bytes */
      data += 1;
      count -= 4;
            
      /* Do we have enough data for a complete map entry of 16 bytes ?
       * program: 4 bytes
       * version: 4 bytes
       * protocol: 4 bytes
       * port: 4 bytes
       */

      if ( count >= 16 ) {
	/* process one complete map entry */
	RPC_PDEBUG_L3("--- parsed complete map entry\n");

	map_program=IXDR_GET_INT32(data); data++;
	RPC_PDEBUG_L3("map_program=0x%x\n",map_program);

	map_version=IXDR_GET_INT32(data); data++;
	RPC_PDEBUG_L3("map_version=0x%x\n",map_version);

	map_protocol=IXDR_GET_INT32(data); data++;
	RPC_PDEBUG_L3("map_protocol=0x%x\n",map_protocol);

	map_port=IXDR_GET_INT32(data);data++;
	RPC_PDEBUG_L3("map_port=0x%x\n",map_port);

	/* compute fate of map entry */
	if (match_rpcs((char *)&rpcinfo->c_procs,rpcinfo->i_procs,map_program)) {
	  /* program is authorized */
	  RPC_PDEBUG_L1("map entry contains authorized program request[%u]\n",
			(unsigned int)map_program);
	  add_expect_related(carrier_protocol,map_protocol,map_port,dir,ct,new_seqnum - count);
	} else {
	  RPC_PDEBUG_L1("map entry contains illegal program request [%u]. [drop]\n",
			(unsigned int)map_program);
	}

	count -= 16;

	/* Did we reach the end of data ? */
	if ( count == 0 ) {
	  /* A valid case.
	   * The data ends exactly with the end of the map.
	   * Due to RFC 1014 XDR there must follow another RM fragment since
	   * at least the final 'end of list' is missing.
	   * Therefore, we have to set our flags accordingly.
	   */
	  RPC_PDEBUG_L3("data in RM fragment ends with complete map entry\n");
	  RPC_PDEBUG_L3("expecting another subsequent RM fragment\n");
	  /* We miss 0 bytes of the map entry
	   *
	   * set FLAGS_MAPFATE       to MAPFATE_UNDEF
	   * set FLAGS_PROTO         to PROTO_UNDEF
	   * set FLAGS_MISSING_BYTES to 0
	   */
	  *new_flags = 
	    /* the part we leave untouched */
	    ((RPC_CT_FLAGS_VALID_MASK | RPC_CT_FLAGS_INITIAL_RM_MASK) & *new_flags )
	    |
	    /* the part we like to set */
	    RPC_CT_FLAGS_MAPFATE_UNDEF | RPC_CT_FLAGS_PROTO_UNDEF | 0;
	} else if (count < 4) {
	  /* This is an error situation */
	  RPC_PERROR("map entry starts with only %d bytes\n",count);
	  /* invalidate */
	  *new_flags = ( *new_flags & ~(RPC_CT_FLAGS_VALID_YES));
	  /* Exit the function here */
	  return RPC_PKG_NOMATCH;
	}
	/* End of a loop cycle:
	 * Either count == 0:
	 *    This ends the while loop.
	 *    We set the flags accordingly above.
	 * Or count >=4
	 *    We are positioned at the start of a next entry and
	 *    have at least 4 bytes left.
	 *    Continue the while loop.
	 */
      } else {
	/* Partial entry; less than 16 bytes left */
	RPC_PDEBUG_L3("--- parsing %d bytes of partial map entry\n", count);
	/* From our understanding of the RFC 1050 we deduce that
	 * we can end up here with either 0,4,8 or 12 bytes left.
	 * Check this:
	 */
	switch (count) {
	case 0:
	  /* 0 bytes left 
	   * we miss: program, version, protocol, port
	   *
	   * set FLAGS_MAPFATE       to MAPFATE_UNKNOWN
	   * set FLAGS_PROTO         to PROTO_UNDEF
	   * set FLAGS_MISSING_BYTES to 16
	   */
	  *new_flags = 
	    /* the part we leave untouched */
	    ((RPC_CT_FLAGS_VALID_MASK | RPC_CT_FLAGS_INITIAL_RM_MASK) & *new_flags )
	    |
	    /* the part we like to set */
	    RPC_CT_FLAGS_MAPFATE_UNKNOWN | RPC_CT_FLAGS_PROTO_UNDEF | 
	    (RPC_CT_FLAGS_MISSING_BYTES_MASK & (u_int16_t) 16);
	  break;
	case 4:
	  /* 4 bytes left
	   * we have: program
	   * we miss: version, protocol, port
	   *
	   * set FLAGS_MAPFATE       according to result of match_rpcs
	   * set FLAGS_PROTO         to PROTO_UNDEF
	   * set FLAGS_MISSING_BYTES to 12
	   */

	  map_program=IXDR_GET_INT32(data); data++;
	  RPC_PDEBUG_L3("map_program=0x%x\n",map_program);

	  /* compute fate of map entry */
	  mapfate_flag = match_rpcs((char *)&rpcinfo->c_procs,rpcinfo->i_procs,
				    map_program) ?
	    RPC_CT_FLAGS_MAPFATE_PASS : RPC_CT_FLAGS_MAPFATE_DROP;

	  *new_flags = 
	    /* the part we leave untouched */
	    ((RPC_CT_FLAGS_VALID_MASK | RPC_CT_FLAGS_INITIAL_RM_MASK) & *new_flags )
	    |
	    /* the part we like to set */
	    mapfate_flag | RPC_CT_FLAGS_PROTO_UNDEF | 
	    (RPC_CT_FLAGS_MISSING_BYTES_MASK & (u_int16_t) 12);
	  break;
	case 8:
	  /* 8 bytes left
	   * we have: program, version
	   * we miss: protocol, port
	   *
	   * set FLAGS_MAPFATE       according to result of match_rpcs
	   * set FLAGS_PROTO         to PROTO_UNDEF
	   * set FLAGS_MISSING_BYTES to 8
	   */

	  map_program=IXDR_GET_INT32(data); data++;
	  RPC_PDEBUG_L3("map_program=0x%x\n",map_program);

	  map_version=IXDR_GET_INT32(data); data++;
	  RPC_PDEBUG_L3("map_version=0x%x\n",map_version);

	  /* compute fate of map entry */
	  mapfate_flag = match_rpcs((char *)&rpcinfo->c_procs,rpcinfo->i_procs,
				    map_program) ?
	    RPC_CT_FLAGS_MAPFATE_PASS : RPC_CT_FLAGS_MAPFATE_DROP;
                        
	  *new_flags = 
	    /* the part we leave untouched */
	    ((RPC_CT_FLAGS_VALID_MASK | RPC_CT_FLAGS_INITIAL_RM_MASK) & *new_flags )
	    |
	    /* the part we like to set */
	    mapfate_flag | RPC_CT_FLAGS_PROTO_UNDEF | 
	    (RPC_CT_FLAGS_MISSING_BYTES_MASK & (u_int16_t) 8);
	  break;
	case 12:
	  /* 12 bytes left
	   * we have: program, version, protocol
	   * we miss: port
	   *
	   * set FLAGS_MAPFATE       according to result of match_rpcs
	   * set FLAGS_PROTO         according to the available data
	   * set FLAGS_MISSING_BYTES to 4
	   */

	  map_program=IXDR_GET_INT32(data); data++;
	  RPC_PDEBUG_L3("map_program=0x%x\n",map_program);
                        
	  map_version=IXDR_GET_INT32(data); data++;
	  RPC_PDEBUG_L3("map_version=0x%x\n",map_version);

	  map_protocol=IXDR_GET_INT32(data); data++;
	  RPC_PDEBUG_L3("map_protocol=0x%x\n",map_protocol);

	  /* compute fate of map entry */
	  mapfate_flag = match_rpcs((char *)&rpcinfo->c_procs,rpcinfo->i_procs,
				    map_program) ?
	    RPC_CT_FLAGS_MAPFATE_PASS : RPC_CT_FLAGS_MAPFATE_DROP;

	  if (map_protocol == IPPROTO_UDP ) {
	    proto_flag = RPC_CT_FLAGS_PROTO_UDP;
	  } else if (map_protocol == IPPROTO_TCP ) {
	    proto_flag = RPC_CT_FLAGS_PROTO_TCP;
	  } else {
	    RPC_PERROR("protocol 0x%x inside a map entry is not supported by matcher\n",
		       map_protocol);
	    proto_flag = RPC_CT_FLAGS_PROTO_UNDEF;
	    mapfate_flag = RPC_CT_FLAGS_MAPFATE_DROP;
	  }

	  *new_flags = 
	    /* the part we leave untouched */
	    ((RPC_CT_FLAGS_VALID_MASK | RPC_CT_FLAGS_INITIAL_RM_MASK) & *new_flags )
	    |
	    /* the part we like to set */
	    mapfate_flag | proto_flag | 
	    (RPC_CT_FLAGS_MISSING_BYTES_MASK & (u_int16_t) 4);
	  break;
	default:
	  RPC_PERROR("Count of %d bytes left is not in line with RFC 1050\n", count);
	  /* invalidate */
	  *new_flags = ( *new_flags & ~(RPC_CT_FLAGS_VALID_YES));
	  /* Exit the function here */
	  return RPC_PKG_NOMATCH;
	}
	/* End of a loop cycle:
	 * We processed the partial map entry.
	 * 
	 * Either count == 0,4,8,12: these are valid cases.
	 *    We already set the flags accordingly above.
	 * Or count%4 != 0
	 *    We processed that error case above and did a direct exit
	 */

	/* Set the counter count to 0 to force end of loop */
	count = 0;

	RPC_PDEBUG_L3("data in RM fragment ends with partial map entry\n");
	RPC_PDEBUG_L3("expecting another subsequent RM fragment\n");
      }
    } /* value follows was signaled*/
  } /* end of while (count > 0) */

  /* We scanned the map to the end of the RM fragment without problems. 
   * Our verdict is match.
   */
  return RPC_PKG_MATCH; 
}

static int check_rpc_packet_call(const u_int32_t *data, const void *matchinfo,
				 int *hotdrop, int dir, struct ip_conntrack *ct,
				 int offset,u_int16_t len_rpc_procdata)
{
  const struct ipt_rpc_info *rpcinfo = matchinfo;

  /* Note:
   * Currently, we did not register our helper structure (struct ip_ct_rpc_master)
   * in union ip_conntrack_help (file ip_conntrack.h).
   * However, we check whether our helper structure needs less or equal size than
   * the officialy registered helper sturcture ip_ct_ftp_master. See the init function of the module.
   */
  struct ip_ct_rpc_master *ct_rpc_info = (struct ip_ct_rpc_master *) &ct->help.ct_ftp_info;

  struct request_p *req_p=0;
  u_int32_t xid;

  u_int32_t prog_num;
  u_int32_t prog_num_inside;
  u_int32_t proc_num;

  /* Get XID */
  xid = *data;

  /* We initialize the main ct_rpc_info flags.
   * Note: we use the main flags both for TCP and UDP in order to  check
   * for a proper protocol flow (call -> reply for xid).
   */
  LOCK_BH(&ipct_rpc_lock);
  ct_rpc_info->bitfield[!dir] = (
				 RPC_CT_FLAGS_VALID_YES |
				 RPC_CT_FLAGS_INITIAL_RM_YES |
				 RPC_CT_FLAGS_MAPFATE_UNDEF |
				 RPC_CT_FLAGS_PROTO_UNDEF
				 );
  /* For the first reply we transport the xid from the call via the next_seq_num field */
  ct_rpc_info->next_seq_num[!dir] = xid;
  UNLOCK_BH(&ipct_rpc_lock);

  RPC_PDEBUG_L2("set ct_rpc_info xid=%x\n", xid);

  /* Get the RPC program number used in this call */
  data += 3;
  prog_num = IXDR_GET_INT32(data);

  /* Get the RPC procedure number used in this call */
  data += 2;
  proc_num = IXDR_GET_INT32(data);

  if ( prog_num == 100000 && proc_num == 3 ) {
    RPC_PDEBUG_L1("detected RPC call to program number\n");
    RPC_PDEBUG_L1("portmapper [%u], procedure GETPORT [%u]\n",
		  prog_num, proc_num);
    /* For the normal portmapper GETPORT we extract the requested protocol
     * as well
     * Advance pointer to the Credentials */
    data++;

    /* Jump Credentials and Verifier */
    data += 2;
    data += 2;

    /* Do an additional length check.
     * Data beyond the verifier are not guaranteed!
     * However we passed the length of the procedure payload after the verifier.
     */
    if ( len_rpc_procdata != 16 )  {
      if (rpcinfo->strict == 1) {
	*hotdrop = 1;
	RPC_PDEBUG_L2("packet length is incorrect for RPC GETPORT call. [drop]\n");
      } else {
	RPC_PDEBUG_L2("packet length is incorrect for RPC GETPORT call. [skip]\n");
      }
      return RPC_PKG_NOMATCH;
    }

    /* Get and print RPC program number inside the GETPORT query */
    prog_num_inside = (unsigned int)IXDR_GET_INT32(data);
    RPC_PDEBUG_L2("RPC call contains a query for program [%u]. [check]\n",
		  prog_num_inside);
        
    /* Check if the prog_num_inside is an allowed program to query for
     * Function match_rpcs returns 0 if the query is not allowed.
     */
    if (match_rpcs((char *)&rpcinfo->c_procs,
		   rpcinfo->i_procs, prog_num_inside) == 0) {
      RPC_PDEBUG_L1("RPC packet contains illegal program request [%u]. [cont]\n",
		    (unsigned int)prog_num_inside);

      /* If the RPC conntrack half entry already exists .. */

      switch (ct->tuplehash[dir].tuple.dst.protonum) {
      case IPPROTO_UDP:
	WRITE_LOCK(&ipct_rpc_udp_lock);
	req_p = LIST_FIND(&request_p_list_udp, request_p_cmp_kind_getport,
			  struct request_p *, xid,
			  ct->tuplehash[dir].tuple.src.ip);
	break;
      case IPPROTO_TCP:
	WRITE_LOCK(&ipct_rpc_tcp_lock);
	req_p = LIST_FIND(&request_p_list_tcp, request_p_cmp_kind_getport,
			  struct request_p *, xid,
			  ct->tuplehash[dir].tuple.src.ip);
	break;
      }

      if (req_p) {
	RPC_PDEBUG_L2("found req_p for xid=%x proto=%u ip=%u.%u.%u.%u\n",
		      xid, ct->tuplehash[dir].tuple.dst.protonum,
		      NIPQUAD(ct->tuplehash[dir].tuple.src.ip));
                
	/* We remove the request and therefore inhibit the helper
	 *   to associate the reply
	 */
	if (del_timer(&req_p->timeout))
	  req_p->timeout.expires = 0;


	switch (ct->tuplehash[dir].tuple.dst.protonum) {
	case IPPROTO_UDP:
	  LIST_DELETE(&request_p_list_udp, req_p);
	  RPC_PDEBUG_L2("RPC req_p removed. [done]\n");
	  WRITE_UNLOCK(&ipct_rpc_udp_lock);
	  break;
	case IPPROTO_TCP:
	  LIST_DELETE(&request_p_list_tcp, req_p);
	  RPC_PDEBUG_L2("RPC req_p removed. [done]\n");
	  WRITE_UNLOCK(&ipct_rpc_tcp_lock);
	  break;
	}

	/* free kernel memory of deleted list entry */
	kfree(req_p);

      } else {
	/* FABR: this should never happen.
	 * A packet that the matcher sees should always have passed the helper before!
	 *
	 * This could only happen if a connection to a portmapper was itsself 
	 * expected. Then a subsequent connection (e.g. GETPORT) to this portmapper 
	 * would already be 'RELATED' and therefore would not pass the helper module.
	 * However, it would pass the matcher module and we get here.
	 * We block this kind of situation at all places where we add expectations.
	 * We inhibit explicitly to add an expectation for a portmapper port that we supervise.
	 */
	RPC_PWARN("no req_p found for xid=%x proto=%u ip=%u.%u.%u.%u\n",
		  xid, ct->tuplehash[dir].tuple.dst.protonum,
		  NIPQUAD(ct->tuplehash[dir].tuple.src.ip));
	switch (ct->tuplehash[dir].tuple.dst.protonum) {
	case IPPROTO_UDP:
	  WRITE_UNLOCK(&ipct_rpc_udp_lock);
	  break;
	case IPPROTO_TCP:
	  WRITE_UNLOCK(&ipct_rpc_tcp_lock);
	  break;
	}
      }

      /* If strict is set we not only deny our match verdict but we
       * require to drop the packet immediately
       */
      if(rpcinfo->strict == 1) {
	*hotdrop = 1;
	RPC_PDEBUG_L2("RPC packets illegal program request removed. [drop]\n");
      } else {
	RPC_PDEBUG_L2("RPC packets illegal program request removed. [skip]\n");
      }
      return RPC_PKG_NOMATCH;
    }

    RPC_PDEBUG_L1("RPC call contains authorised query for program [%u]. [match]\n",
		  (unsigned int)prog_num_inside);
    return RPC_PKG_MATCH;

  } else if ( prog_num == 390512 && proc_num == 3 ) {
    RPC_PDEBUG_L1("detected proprietary ClearCase RPC call to\n");
    RPC_PDEBUG_L1("program number [%u], procedure GETPORT [%u].\n",
		  prog_num, proc_num);

    /* We always let RPC calls to the ALBD daemon pass
     * since we cannot check for shares or alike contained inside the query.
     * -> It is in vain to use matching rules that specify --rpcs for the ALBD portmapper. 
     */

    RPC_PDEBUG_L1("queries to the ALBD are always authorized. [match]\n");
    return RPC_PKG_MATCH;

  } else if (prog_num == 100000 && proc_num == 4) {
    RPC_PDEBUG_L1("detected RPC call to portmapper [%u], procedure DUMP [%u]\n",
		  prog_num, proc_num);

    /* We always let RPC calls to the portmapper procedure DUMP pass */

    RPC_PDEBUG_L1("DUMP queries to the portmapper are always authorized. [match]\n");
    return RPC_PKG_MATCH;
        
  } else {
    /* we detected a RPC call packet to one of the supervised portmappers that is neither
     *    prog 100000 proc 3   nor
     *    prog 100000 proc 4   nor
     *    prog 390512 proc 3
     *
     * The fact that we see it at all means that there is no state for that
     * connection.
     * We are not interested in this kind of packets and therefore do not
     * let them slip by our own vote.
     * Our verdict is therefore RPC_PKG_NOMATCH and we rely on the fact
     * that some rule behind us will match this packet via NEW
     */
    RPC_PWARN("ignoring RPC call to program [%u] and procedure [%u]. [skip]\n",
	      prog_num, proc_num);

    return RPC_PKG_NOMATCH;
  }
}

static int check_rpc_packet_reply_tcp(const u_int32_t *data, const void *matchinfo,
				      int *hotdrop, int dir, struct ip_conntrack *ct,
				      int offset, u_int16_t datalen, u_int32_t tcp_seqnum)
{
  struct ip_ct_rpc_master *ct_rpc_info = (struct ip_ct_rpc_master *) &ct->help.ct_ftp_info;
  u_int16_t old_flags, new_flags;
  u_int32_t old_seqnum, new_seqnum;

  u_int32_t xid;

  /* variables for the RM fragment header */
  u_int32_t rm_fragment_header;
  int is_last_rm_fragment;
  u_int32_t rm_fragment_body_len;

  /* Return value of the function.
   * We use it to accumulate the return value unless we leave the function preemptively
   */
  int verdict;


  /* Read the context we stored in the ct */
  LOCK_BH(&ipct_rpc_lock);
  /* read the old flags*/
  old_flags = ct_rpc_info->bitfield[dir];
  RPC_PDEBUG_L3("old RPC_CT_FLAGS=%x\n",old_flags);
  /* read the old tcp sequence number rsp. xid in case of an initial RM fragment */
  old_seqnum = ct_rpc_info->next_seq_num[dir];
  UNLOCK_BH(&ipct_rpc_lock);

  /* Check for the flag RPC_CT_FLAGS_VALID_YES
   * At this point we never expect to see a packet without RPC_CT_FLAGS_VALID_YES
   */
  if ( ! (old_flags & RPC_CT_FLAGS_VALID_YES) ) {
    RPC_PERROR("reply packet signals RPC_CT_FLAGS_VALID_NO. [skip]\n");
    /* early exit */
    return RPC_PKG_NOMATCH;
  }

  /* So we got a reply packet for which we saw the call already (RPC_CT_FLAGS_VALID_YES).
   * Due to our checks in the caller of this function it is ensured that the packet
   * contains TCP payload. Therefore, we have to check and maintain the TCP sequence number below.
   */
     
  /* Parse and strip the RM fragment header first */
  /* parse the RM header */
  rm_fragment_header = IXDR_GET_INT32(data);
  is_last_rm_fragment = ((int) (rm_fragment_header & 0x80000000) == 0x80000000);
  rm_fragment_body_len = (u_int32_t) (rm_fragment_header & 0x7fff);

  RPC_PDEBUG_L2("rm_fragment_header=%x\n",rm_fragment_header);
  RPC_PDEBUG_L2("is_last_rm_fragment=%d\n",is_last_rm_fragment);
  RPC_PDEBUG_L2("rm_fragment_body_len=0x%x\n",rm_fragment_body_len);

  /* Do some additional length check:
   * len(rm_fragment_header) = 4
   * len(tcp payload) = datalen
   * The rm_fragment_body_len + len(rm_fragment_header) == len(tcp payload)
   */

  if ( rm_fragment_body_len + 4 != datalen) {
    RPC_PERROR("Length of TCP payload 0x%x not equal to reported length via RM protocol 0x%x\n",
	       datalen,
	       rm_fragment_body_len + 4);
    /* early exit */
    return RPC_PKG_NOMATCH;
  }

  /* Additionaly, do length checks. If this cannot be a DUMP reply, bail out */
  if ( datalen == 32 || datalen == 68 ) {
    RPC_PDEBUG_L2("this is some TCP GETPORT reply. [skip]\n");
    /* early exit */
    return RPC_PKG_NOMATCH;
  }

  /* advance the data pointer to the RM fragment body */
  data++;

  /* What follows is the last conditional inside this function.
   * We distinguish initial and subsequent RM fragments.
   * In order to avoid deep nesting of inner conditionals we decided
   * to use a single exit point per branch.
   * We use a goto-label approach here.
   */

  /* Initial reply packets are treated differently than follow up packets */
  if ( (old_flags & RPC_CT_FLAGS_INITIAL_RM_YES) ) {
    /* On this branch we exit the function through a jump to
     * label exit_from_initial_fragment
     */

    /* initialize the data we need at the exit point */
    verdict = RPC_PKG_NOMATCH;   /* default is a no-match */
    new_flags = old_flags;       /* default is no change for flags */
    new_seqnum = tcp_seqnum;     /* default is the current one */

    /* This is some initial RM reply fragment */
    RPC_PDEBUG_L3("Xid in ct context=%x\n",old_seqnum);

    /* Check the xid of the packet against the xid stored in field next_seq_num[dir]
     * This is only valid and possible for initial RM fragments.
     *
     * Note: xid is stored in network order in the packets and our data structures.
     *       This is true as well for the piggy back storage in slot
     *       ct_rpc_info->next_seq_num[dir] which is old_seqnum in the context below.
     *       We also print it without recoding into host order.
     *       Since we never do any computations with the xid this allows us to save
     *       the processing time for the reoding via ntohl(x)
     */
    xid = *data;
    if ( xid != old_seqnum) {
      RPC_PERROR("Xid of initial reply packet %x does not match stored Xid in ct %x. [skip]\n",
		 xid,
		 old_seqnum);

      /* invalidate this thread; clear the flag RPC_CT_FLAGS_VALID_YES */
      new_flags = ( new_flags & ~(RPC_CT_FLAGS_VALID_YES));
      /* a more brutal variant would be: new_flags = RPC_CT_FLAGS_VALID_NO */

      verdict=RPC_PKG_NOMATCH;
      goto exit_from_initial_fragment;
    } 

    RPC_PDEBUG_L2("initial RM of RPC DUMP reply was expected by matcher. [cont]\n");
    RPC_PDEBUG_L2("xid=%x proto=%u ip=%u.%u.%u.%u\n",
		  xid, ct->tuplehash[!dir].tuple.dst.protonum,
		  NIPQUAD(ct->tuplehash[!dir].tuple.src.ip));
        
    /* So we got a matching reply packet that contains data
     * Update flags and sequence number
     */

    /* clear flag RPC_CT_FLAGS_INITIAL_RM_YES */
    new_flags = ( new_flags & ~(RPC_CT_FLAGS_INITIAL_RM_YES));

    /* Initialize TCP sequence number: we are reading the first data of the tcp payload.
     * The parameter datalen contains the len of tcp payload without tcp header.
     * The parameter tcp_seqnum contains the tcp sequence number of the current packet in
     * host order (see caller of this function)!
     * In slot ct_rpc_info->next_seq_num[dir] we store the predicted sequence number in host order!
     * Reason: we need to do some computation on the values at several positions in this function.
     *         By storing the value in hostorder we need only to compute ntohl(x) once when
     *         our function is called.
     */
    new_seqnum = tcp_seqnum + datalen;
    RPC_PDEBUG_L3("current TCP seqnum=%x\n",tcp_seqnum);
    RPC_PDEBUG_L3("TCP datalen=0x%x\n",datalen);
        
    /* Advance to the field MsgType of the RPC header   */
    data++;
    /* Verify if packet is really an RPC reply packet */
    if (IXDR_GET_INT32(data) != 1) {
      RPC_PDEBUG_L2("The MsgType of the packet is not 'reply'. [skip]\n");
      verdict=RPC_PKG_NOMATCH;
      goto exit_from_initial_fragment;
    }

    /* Advance to the field ReplyState of the RPC body  */
    data++;
    /* Is status accept? */
    if (IXDR_GET_INT32(data)) {
      RPC_PDEBUG_L2("The ReplyState of the packet is not 'accept'. [skip]\n");
      verdict=RPC_PKG_NOMATCH;
      goto exit_from_initial_fragment;
    }

    /* Advance to the field Verifier of the RPC body  */
    data++;
    /* Advance to the field AcceptStatus of the RPC body  */
    data += 2;
    /* Is accept status "success"? */
    if (IXDR_GET_INT32(data)) {
      RPC_PDEBUG_L2("The AcceptStatus of the packet is not 'success'. [skip]\n");
      verdict=RPC_PKG_NOMATCH;
      goto exit_from_initial_fragment;
    }

    /* advance to the procedure specific result. In our case the begin of the map list */
    data++;

    /* Call the scanner for the map list
     *
     * Since this is an initial RM fragment the RM fragment body contains a header
     *           Xid: 4 bytes
     *       MsgType: 4 bytes
     *    ReplyState: 4 bytes
     *      Verifier: 8 bytes
     *   AcceptState: 4 bytes
     *
     * In sum this is 24 bytes.
     */

    verdict = scan_map_list(data,matchinfo,dir,ct,IPPROTO_TCP,
			    rm_fragment_body_len - 24,
			    new_seqnum,
			    old_flags,
			    &new_flags 
			    );

  exit_from_initial_fragment:
    /* at this point the following variables must be set:
     * verdict: exit value of function
     * new_seqnum: the new tcp sequence number we like to store in the ct context
     * new_flags: the new flags we like to store in the ct context
     *
     */

    /* set the accumulated new flags and data */
    LOCK_BH(&ipct_rpc_lock);
    ct_rpc_info->bitfield[dir] = new_flags;
    ct_rpc_info->next_seq_num[dir] = new_seqnum;
    UNLOCK_BH(&ipct_rpc_lock);

    RPC_PDEBUG_L3("new RPC_CT_FLAGS=%x\n",new_flags);
    RPC_PDEBUG_L3("new predicted seqnum=%x\n",new_seqnum);

    RPC_PDEBUG_L1("initial RM fragment of RPC reply processed. [match]\n");
    return verdict;
        
  } else {
    /* On this branch we exit the function through a jump to
     * label exit_from_subsequent_fragment
     */

    /* initialize the data we need at the exit point */
    verdict = RPC_PKG_NOMATCH;   /* default is a no-match */
    new_flags = old_flags;       /* default is no change for flags */
    new_seqnum = tcp_seqnum;     /* default is the current one */

    /* This is some follow up RM reply fragment */
    RPC_PDEBUG_L3("old predicted seqnum=%x\n",old_seqnum);

    /* Check the TCP sequence number */
    if ( tcp_seqnum != old_seqnum ) {
      RPC_PWARN("wrong TCP seqnum: stored=%x seen=%x\n",
		old_seqnum,
		tcp_seqnum);
      goto exit_from_subsequent_fragment;
    }

    /* Sequence number is OK: Start processing the subsequent RM fragment */
    RPC_PDEBUG_L2("detected subsequent RM of RPC DUMP reply with correct seqnum. [cont]\n");
        
    /* Update TCP sequence number
     * Note: We checked that the current sequence number == predicted one
     *       We got datalen > 0
     *       -> after(tcp_seqnum + datalen, old_seqnum) == 1
     */
    new_seqnum = tcp_seqnum + datalen;

    /* Since this is a subsequent RM fragment the data pointer is already
     * positioned at the beginning of the (fragmented) map list
     */

    /* Call the scanner for the map list */

    verdict = scan_map_list(data,matchinfo,dir,ct,IPPROTO_TCP,
			    rm_fragment_body_len,
			    new_seqnum,
			    old_flags,
			    &new_flags 
			    );

  exit_from_subsequent_fragment:
    /* at this point the following variables must be set:
     * verdict: exit value of function
     * new_seqnum: the new tcp sequence number we like to store in the ct context
     * new_flags: the new flags we like to store in the ct context
     *
     */

    /* set the accumulated new flags and data */
    LOCK_BH(&ipct_rpc_lock);
    ct_rpc_info->bitfield[dir] = new_flags;
    ct_rpc_info->next_seq_num[dir] = new_seqnum;
    UNLOCK_BH(&ipct_rpc_lock);

    RPC_PDEBUG_L3("new RPC_CT_FLAGS=%x\n",new_flags);
    RPC_PDEBUG_L3("new predicted seqnum=%x\n",new_seqnum);

    RPC_PDEBUG_L1("subsequent RM fragment of DUMP reply processed. [match]\n");
    return verdict;
  }
  /* This point of the function can never be reached */
}

static int check_rpc_packet_reply_udp(const u_int32_t *data, const void *matchinfo,
				      int *hotdrop, int dir, struct ip_conntrack *ct,
				      int offset, u_int16_t datalen)
{
  struct ip_ct_rpc_master *ct_rpc_info = (struct ip_ct_rpc_master *) &ct->help.ct_ftp_info;
  u_int16_t old_flags, new_flags;
  u_int32_t old_seqnum;

  u_int32_t xid;

  /* Return value of the function.
   * We use it to accumulate the return value unless we leave the function preemptively
   */
  int verdict;


  /* Read the context we stored in the ct */
  LOCK_BH(&ipct_rpc_lock);
  /* read the old flags*/
  old_flags = ct_rpc_info->bitfield[dir];
  RPC_PDEBUG_L3("old RPC_CT_FLAGS=%x\n",old_flags);
  /* read the xid we stored in the ct private context
   * Note: UPD DUMP replies do not contain RM fragments! 
   */
  old_seqnum = ct_rpc_info->next_seq_num[dir];
  UNLOCK_BH(&ipct_rpc_lock);

  /* Check for the flag RPC_CT_FLAGS_VALID_YES
   * At this point we never expect to see a packet without RPC_CT_FLAGS_VALID_YES
   */
  if ( ! (old_flags & RPC_CT_FLAGS_VALID_YES) ) {
    RPC_PERROR("reply packet signals RPC_CT_FLAGS_VALID_NO. [skip]\n");
    /* early exit */
    return RPC_PKG_NOMATCH;
  }

  /* So we got a reply packet for which we saw the call already (RPC_CT_FLAGS_VALID_YES).
   * Due to our checks in the caller of this function it is ensured that the packet
   * contains UPD payload.
   */
     
  /* Additionaly, do length checks. If this cannot be a DUMP reply, bail out */
  if ( datalen == 28 || datalen == 64 ) {
    RPC_PDEBUG_L2("this is some UDP GETPORT reply. [skip]\n");
    /* early exit */
    return RPC_PKG_NOMATCH;
  }

  /*
   * We use a goto-label approach here.
   * We exit the function through a jump to
   * label exit_from_rpc_reply
   */

  /* initialize the data we need at the exit point */
  verdict = RPC_PKG_NOMATCH;   /* default is a no-match */
  new_flags = old_flags;       /* default is no change for flags */

  /* This is a reply packet containing the Xid (this is UDP) */
  RPC_PDEBUG_L3("Xid in ct context=%x\n",old_seqnum);

  /* Check the xid of the packet against the xid stored in field next_seq_num[dir]
   *
   * Note: xid is stored in network order in the packets and our data structures.
   *       This is true as well for the piggy back storage in slot
   *       ct_rpc_info->next_seq_num[dir] which is old_seqnum in the context below.
   *       We also print it without recoding into host order.
   *       Since we never do any computations with the xid this allows us to save
   *       the processing time for the reoding via ntohl(x)
   */
  xid = *data;
  if ( xid != old_seqnum) {
    RPC_PERROR("Xid of reply packet %x does not match stored Xid in ct %x. [skip]\n",
	       xid,
	       old_seqnum);

    /* invalidate this thread; clear the flag RPC_CT_FLAGS_VALID_YES */
    new_flags = ( new_flags & ~(RPC_CT_FLAGS_VALID_YES));
    /* a more brutal variant would be: new_flags = RPC_CT_FLAGS_VALID_NO */

    verdict=RPC_PKG_NOMATCH;
    goto exit_from_rpc_reply;
  } 

  RPC_PDEBUG_L2("RPC DUMP reply was expected by matcher. [cont]\n");
  RPC_PDEBUG_L2("xid=%x proto=%u ip=%u.%u.%u.%u\n",
		xid, ct->tuplehash[!dir].tuple.dst.protonum,
		NIPQUAD(ct->tuplehash[!dir].tuple.src.ip));
        
  /* So we got a matching reply packet that contains data
   * Update flags and sequence number
   */

  /* clear flag RPC_CT_FLAGS_INITIAL_RM_YES */
  new_flags = ( new_flags & ~(RPC_CT_FLAGS_INITIAL_RM_YES));

  /* We do not have to tend a TCP sequence number here. This is UDP. */
        
  /* Advance to the field MsgType of the RPC header   */
  data++;
  /* Verify if packet is really an RPC reply packet */
  if (IXDR_GET_INT32(data) != 1) {
    RPC_PDEBUG_L2("The MsgType of the packet is not 'reply'. [skip]\n");
    verdict=RPC_PKG_NOMATCH;
    goto exit_from_rpc_reply;
  }

  /* Advance to the field ReplyState of the RPC body  */
  data++;
  /* Is status accept? */
  if (IXDR_GET_INT32(data)) {
    RPC_PDEBUG_L2("The ReplyState of the packet is not 'accept'. [skip]\n");
    verdict=RPC_PKG_NOMATCH;
    goto exit_from_rpc_reply;
  }

  /* Advance to the field Verifier of the RPC body  */
  data++;
  /* Advance to the field AcceptStatus of the RPC body  */
  data += 2;
  /* Is accept status "success"? */
  if (IXDR_GET_INT32(data)) {
    RPC_PDEBUG_L2("The AcceptStatus of the packet is not 'success'. [skip]\n");
    verdict=RPC_PKG_NOMATCH;
    goto exit_from_rpc_reply;
  }

  /* advance to the procedure specific result. In our case the begin of the map list */
  data++;

  /* Call the scanner for the map list
   *
   * Size of RPC reply header ahead of a UPD procedure reply payload:
   *           Xid: 4 bytes
   *       MsgType: 4 bytes
   *    ReplyState: 4 bytes
   *      Verifier: 8 bytes
   *   AcceptState: 4 bytes
   *
   * In sum this is 24 bytes.
   */

  verdict = scan_map_list(data,matchinfo,dir,ct,IPPROTO_UDP,
			  datalen - 24,
			  0, /* dummy seq number for UDP */
			  old_flags,
			  &new_flags 
			  );

 exit_from_rpc_reply:
  /* at this point the following variables must be set:
   * verdict: exit value of function
   * new_flags: the new flags we like to store in the ct context
   *
   */

  /* We invalidate this thread since UDP has only one reply packet.
   * Clear the flag RPC_CT_FLAGS_VALID_YES
   */
  new_flags = ( new_flags & ~(RPC_CT_FLAGS_VALID_YES));
  /* a more brutal variant would be: new_flags = RPC_CT_FLAGS_VALID_NO */

  /* set the accumulated new flags and data */
  LOCK_BH(&ipct_rpc_lock);
  ct_rpc_info->bitfield[dir] = new_flags;
  ct_rpc_info->next_seq_num[dir] = 0; /* dummy seq number for UDP */
  UNLOCK_BH(&ipct_rpc_lock);

  RPC_PDEBUG_L3("new RPC_CT_FLAGS=%x\n",new_flags);
  RPC_PDEBUG_L1("DUMP reply processed. [match]\n");
  return verdict;
}

static int match(const struct sk_buff *skb, const struct net_device *in,
                 const struct net_device *out, const void *matchinfo,
                 int offset, const void *hdr, u_int16_t datalen, int *hotdrop)
     /* see header ip_tables.h for a definition of the protoype
      *
      * const struct sk_buff *skb
      * const struct net_device *in
      * const struct net_device *out
      * const void *matchinfo: data specific for the matcher
      * (see ipt_rpc.h:struct ipt_rpc_info)
      * int offset: fragment offset of the IP packet. Should always be 0 for
      * connection tracking!
      * const void *hdr : points to the first byte of the IP packets payload 
      * u_int16_t datalen: length of the IP packets payload
      * int *hotdrop: set *hotdrop=1 to force immediate packet drop
      */
{
  struct ip_conntrack *ct;
  enum ip_conntrack_info ctinfo;
  const u_int32_t *data;
  enum ip_conntrack_dir dir;
  const struct tcphdr *tcp;
  const struct ipt_rpc_info *rpcinfo = matchinfo;
  int port, portsok;
  int tval;


  RPC_PDEBUG_L2("----- new packet to evaluate -----\n");

  ct = ip_conntrack_get((struct sk_buff *)skb, &ctinfo);
  /* Skip immediately if we do not find any connection tracking
   * information about that packet
   */
  if (!ct) {
    RPC_PDEBUG_L2("no ct available. [skip]\n");
    return RPC_PKG_NOMATCH;
  }
    
  RPC_PDEBUG_L2("ct detected. [cont]\n");

  /* We are only interested in TCP/UDP traffic */
  switch (ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.protonum) {
  case IPPROTO_UDP:
  case IPPROTO_TCP:
    break;
  default:
    RPC_PDEBUG_L2("transport protocol=%u, is of no interest for us [skip]\n",
		  ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.protonum);
    return RPC_PKG_NOMATCH;
  }

  /* Get the direction of the packet:
   * Is it client to server (IP_CT_DIR_ORIGINAL) or server to client
   */
  dir = CTINFO2DIR(ctinfo);

  /* FABR:
   * The RPC calls of kind
   *  REQUEST_P_BLOB_KIND_GETPORT and
   *  REQUEST_P_BLOB_KIND_GETPORT_ALBD
   * are treated when they are sent from client to server (IP_CT_DIR_ORIGINAL)
   * If they are not allowed we remove them immediately from the helpers request
   * list which will inhibit the helper to recognize the reply.
   * As a result the helper will not extract the server port from the answer and
   * ther will be not established any RELATED state.
   *
   * In contrast to that the fate of RPC calls for program portmapper procedure DUMP
   * is decided when the answer with the list of all server ports is sent from
   * the server back to the client. The matcher module is scanning the list
   * of all server ports and is establishing a RELATED state only for those
   * server ports where the related RPC program number is enumerated in the match rule.
   *
   */

  if (dir == IP_CT_DIR_ORIGINAL) {
    /* In this direction we track:
     * - RPC call program number portmapper (100000) procedure GETPORT (3)
     *   This is the standard case
     * - RPC call program number ??? (390512) procedure GETPORT (3)
     *   This is some proprietary RPC call to the albd server of ClearCase.
     *   We track the call in order to learn about the server ports that are
     *   communicated to the initiator. The tracking is derived from the standard case.
     * - RPC call program number portmapper (100000) procedure DUMP (4)
     *   We also check for the RPC call program number portmapper (100000)
     *   procedure DUMP (4).
     *   In contrast to the other calls we do not judge whether the call is allowed.
     *   This would be impossible since the DUMP call does not contain any program
     *   numbers. We only add the call to the request list (of the helper).
     *   This enables us to block any faked DUMP replies from the server side
     *   that where not initiated by the initiaters side.
     * 
     */

    RPC_PDEBUG_L2("packet is from the initiator [cont]\n");

    /* Do some sanity checking on UDP or TCP packets,
     * like their respective helper modules.
     */
    switch (ct->tuplehash[dir].tuple.dst.protonum) {
    case IPPROTO_UDP:
      RPC_PDEBUG_L2("RPC call uses protocol UDP [cont]\n");
      /* do some header checking */
      if (offset != 0) {
	RPC_PWARN("offset of IP packet !=0; no connection tracking. [skip]\n");
	return RPC_PKG_NOMATCH;
      }

      if (datalen < sizeof(struct udphdr)) {
	RPC_PDEBUG_L2("packet does not contain a complete header. [skip]\n");
	return RPC_PKG_NOMATCH;
      }

      /* NOTE: for UDP there are no handshake packets with payload
	 length == 0 */

      /* Check whether the packet is destined for one of the RPC server
       * ports that where specified via the module load command line.
       * Otherwise, we are not interested in the packet.
       */
      for (port=0,portsok=0; port < ports_n_c; port++) {
	if (ntohs(ct->tuplehash[dir].tuple.dst.u.all) == ports[port]) {
	  portsok++;
	  break;
	}
      }

      if (portsok == 0) {
	RPC_PDEBUG_L3("packet is not destined for a portmapper [%u]. [skip]\n",
		      ntohs(ct->tuplehash[dir].tuple.dst.u.all));
	return RPC_PKG_NOMATCH;
      }

      /* OK: we are interested in this packet
       * Do a length check for the UDP payload
       *  A RPC call via UDP consists at least of the following data:
       *  08 bytes RPC call header
       *         04 bytes XID
       *         04 bytes MsgType
       *  32 bytes RPC call body (mandatory prefix)
       *         04 bytes RpcVerion
       *         04 bytes RpcProgramNumber
       *         04 bytes RpcProgramVersion
       *         04 bytes RpcProcedureNumber
       *        >=8 bytes Credentials: length is variable
       *         08 bytes Verifier
       *  In sum this is at least 40 bytes
       *
       *  The rest depends on the program and procedure number. E.g.
       *  - portmapper (100000)/GETPORT (3): 16 bytes
       *             04 bytes QueryRpcProgram
       *             04 bytes QueryRpcVersion
       *             04 bytes QueryProtocol
       *             04 bytes QueryPort
       *  - portmapper (100000)/DUMP    (4):  0 bytes
       *  - albd       (390512)/GETPORT (3):length depends on credentials
       *
       */

      /*
       * Check for a plausible length of the call.
       * Otherwise bail out.
       */

      if ( (datalen - sizeof(struct udphdr)) < 40 ) {
	if (rpcinfo->strict == 1) {
	  *hotdrop = 1;
	  RPC_PDEBUG_L2("packet length is incorrect for RPC call via UDP. [drop]\n");
	} else {
	  RPC_PDEBUG_L2("packet length is incorrect for RPC call via UDP. [skip]\n");
	}
	return RPC_PKG_NOMATCH;
      }
      RPC_PDEBUG_L2("packet length is plausible. [cont]\n");

      /* Position the data pointer to the begin of the RPC data.
       * In case of UDP there is no 4 byte RM fragment header.
       *
       * We have to skip the UDP header which is
       * 8 bytes == 2 increments for a (const u_int32_t *)
       */
      data = (const u_int32_t *)hdr + 2;

      /* Check the RPC data */
      tval = check_rpc_packet_call(data, matchinfo, hotdrop,
				   dir, ct, offset,
				   datalen - sizeof(struct udphdr) - 40);
      return tval;
      break;
                
    case IPPROTO_TCP:
      RPC_PDEBUG_L2("RPC call uses protocol TCP [cont]\n");
      /* do some header checking */
      if (offset != 0) {
	RPC_PWARN("offset of IP packet !=0; no connection tracking. [skip]\n");
	return RPC_PKG_NOMATCH;
      }

      if (datalen < sizeof(struct tcphdr)) {
	RPC_PDEBUG_L2("packet does not contain a complete header. [skip]\n");
	return RPC_PKG_NOMATCH;
      }

      /* Check whether the TCP packet contains payload.
       * The match function is provided with a pointer hdr that points
       * to the start of the data inside the IP packet. Since we are looking at
       * a TCP packet the pointer points to the TCP header.
       */
      tcp = hdr;
      if (datalen == (tcp->doff * 4)) {
	RPC_PDEBUG_L2("packet does not contain any data. [skip]\n");
	/* Variable datalen contains the length of the IP packets payload.  
	 * If this length is equal to the length of the tcp header
	 * (tcp->doff * 4) then the tcp packet contains no payload.
	 *
	 * NOTE: we return a (0==no match) here.
	 * We rely on some ESTABLISHED -j ACCEPT rule behind us.
	 */
	return RPC_PKG_NOMATCH;
      }

      /* Check whether the packet is destined for one of the RPC server
       * ports that where specified via the module load command line.
       * Otherwise, we are not interested in the packet.
       */
      for (port=0,portsok=0; port < ports_n_c; port++) {
	if (ntohs(ct->tuplehash[dir].tuple.dst.u.all) == ports[port]) {
	  portsok++;
	  break;
	}
      }
      if (portsok == 0) {
	RPC_PDEBUG_L3("packet is not destined for a portmapper [%u]. [skip]\n",
		      ntohs(ct->tuplehash[dir].tuple.dst.u.all));
	return RPC_PKG_NOMATCH;
      }

      /* OK: we are interested in this packet
       * Do a length check for the TCP payload
       *  A RPC call via TCP consists at least of the following data:
       *  04 bytes RM header
       *  08 bytes RPC call header
       *         04 bytes XID
       *         04 bytes MsgType
       *  32 bytes RPC call body (mandatory prefix)
       *         04 bytes RpcVerion
       *         04 bytes RpcProgramNumber
       *         04 bytes RpcProgramVersion
       *         04 bytes RpcProcedureNumber
       *        >=8 bytes Credentials
       *         08 bytes Verifier
       *
       *  In sum this is at least 44 bytes
       *
       *  The rest depends on the program and procedure number. E.g.
       *  - portmapper (100000)/GETPORT (3): 16 bytes
       *             04 bytes QueryRpcProgram
       *             04 bytes QueryRpcVersion
       *             04 bytes QueryProtocol
       *             04 bytes QueryPort
       *  - portmapper (100000)/DUMP    (4):  0 bytes
       *  - albd       (390512)/GETPORT (3): length depends on credentials
       *
       */

      /* 
       * Check for a plausible length of the call.
       * Otherwise bail out.
       */

      if ( (datalen - (tcp->doff * 4)) < 44 ) {
	if(rpcinfo->strict == 1) {
	  *hotdrop = 1;
	  RPC_PDEBUG_L2("packet length is invalid for RPC call via TCP. [drop]\n");
	} else {
	  RPC_PDEBUG_L2("packet length is invalid for RPC call via TCP. [skip]\n");
	}
	return RPC_PKG_NOMATCH;
      }
      RPC_PDEBUG_L2("packet length is plausible. [cont]\n");
                
      /* Position the data pointer to the begin of the RPC data.
       * In case of TCP this is the 4 byte RM fragment header.
       * In the case of a RPC call we are not interested in the RM header data.
       *
       * We have to skip the TCP header + the one 32 bit word of the RM header
       */
      data = (const u_int32_t *)tcp + tcp->doff + 1;

      /* Check the RPC data */
      tval = check_rpc_packet_call(data, matchinfo, hotdrop,
				   dir, ct, offset,
				   datalen - (tcp->doff * 4) - 44);

      return tval;
      break;
                
    } /* switch (ct->tuplehash[0].tuple.dst.protonum) */

    RPC_PWARN("transport protocol=%u, is not supported [skip]\n",
	      ct->tuplehash[dir].tuple.dst.protonum);
    return RPC_PKG_NOMATCH;

  } else {
    /* (dir != IP_CT_DIR_ORIGINAL) */
    /* In this direction we track:
     * - RPC reply for the program number portmapper (100000) procedure DUMP (4).
     *
     * We do not check
     * - RPC reply for program number portmapper (100000) procedure GETPORT (3)
     * - RPC reply for program number ??? (390512) procedure GETPORT (3)
     * The tracking of these is completely done by the helper.
     */

    RPC_PDEBUG_L2("packet is from the receiver [cont]\n");

    /* Do some sanity checking on UDP or TCP packets,
     * like their respective helper modules.
     */
    switch (ct->tuplehash[!dir].tuple.dst.protonum) {
    case IPPROTO_UDP:
      RPC_PDEBUG_L2("RPC reply uses protocol UDP [cont]\n");
      /* do some header checking */
      if (offset != 0) {
	RPC_PWARN("offset of IP packet !=0; no connection tracking. [skip]\n");
	return RPC_PKG_NOMATCH;
      }

      if (datalen < sizeof(struct udphdr)) {
	RPC_PDEBUG_L2("packet does not contain a complete header. [skip]\n");
	return RPC_PKG_NOMATCH;
      }

      /* NOTE: for UDP there are no handshake packets with payload
	 length == 0 */

      /* Check whether the packet originates from one of the RPC server
       * ports that where specified via the module load command line.
       * Otherwise, we are not interested in the packet.
       */
      for (port=0,portsok=0; port < ports_n_c; port++) {
	if (ntohs(ct->tuplehash[!dir].tuple.dst.u.all) == ports[port]) {
	  portsok++;
	  break;
	}
      }

      if (portsok == 0) {
	RPC_PDEBUG_L3("packet does not originate from a portmapper [%u]. [skip]\n",
		      ntohs(ct->tuplehash[!dir].tuple.dst.u.all));
	return RPC_PKG_NOMATCH;
      }

      /* OK: we are interested in this packet
       * Do a length check for the UDP payload
       *  A RPC reply via UDP consists at least of the following data:
       *  08 bytes RPC reply header
       *       04 bytes XID
       *       04 bytes MsgType
       *  16 bytes RPC reply body (mandatory prefix)
       *       04 bytes ReplyState
       *       08 bytes Verifier
       *       04 bytes AcceptState
       *
       *  In sum this is 24 bytes
       *
       *  The rest depends on the program and procedure number. E.g.
       *  - portmapper (100000)/GETPORT (3):  4 bytes
       *             04 bytes AnswerPort
       *  - portmapper (100000)/DUMP    (4): arbitrary (depends on the
       *                                     number of registered programs)
       *  - albd       (390512)/GETPORT (3): 40 bytes
       */

      /* 
       * We cannot check here for the exact length!
       */
      if ((datalen - sizeof(struct udphdr)) < 24) {
	if (rpcinfo->strict == 1) {
	  *hotdrop = 1;
	  RPC_PDEBUG_L2("packet length is not plausible for RPC reply via UDP. [drop]\n");
	} else {
	  RPC_PDEBUG_L2("packet length is not plausible for RPC reply via UDP. [skip]\n");
	}
	return RPC_PKG_NOMATCH;
      }
      RPC_PDEBUG_L2("packet length is plausible. [cont]\n");

      /* Position the data pointer to the begin of the RPC data.
       * In case of UDP there is no 4 byte RM fragment header.
       *
       * We have to skip the UDP header which is
       * 8 bytes == 2 increments for a (const u_int32_t *)
       */
      data = (const u_int32_t *)hdr + 2;

      /* Check the RPC data */
      tval = check_rpc_packet_reply_udp(data, matchinfo, hotdrop,
					dir, ct, offset,
					datalen - sizeof(struct udphdr));

      return tval;
      break;
                
    case IPPROTO_TCP:
      RPC_PDEBUG_L2("RPC reply uses protocol TCP [cont]\n");
      /* do some header checking */
      if (offset != 0) {
	RPC_PWARN("offset of IP packet !=0; no connection tracking. [skip]\n");
	return RPC_PKG_NOMATCH;
      }

      if (datalen < sizeof(struct tcphdr)) {
	RPC_PDEBUG_L2("packet does not contain a complete header. [skip]\n");
	return RPC_PKG_NOMATCH;
      }

      /* Check whether the TCP packet contains payload.
       * The match function is provided with a pointer hdr that points
       * to the start of the data inside the IP packet. Since we are looking at
       * a TCP packet the pointer points to the TCP header.
       */
      tcp = hdr;
      if (datalen == (tcp->doff * 4)) {
	RPC_PDEBUG_L2("packet does not contain any data. [skip]\n");
	/* Variable datalen contains the length of the IP packets payload.  
	 * If this length is equal to the length of the tcp header
	 * (tcp->doff * 4) then the tcp packet contains no payload.
	 *
	 * NOTE: we return a (0==no match) here.
	 * We rely on some ESTABLISHED -j ACCEPT rule behind us.
	 */
	return RPC_PKG_NOMATCH;
      }

      /* Check whether the packet originates from one of the RPC server
       * ports that where specified via the module load command line.
       * Otherwise, we are not interested in the packet.
       */
      for (port=0,portsok=0; port < ports_n_c; port++) {
	if (ntohs(ct->tuplehash[!dir].tuple.dst.u.all) == ports[port]) {
	  portsok++;
	  break;
	}
      }

      if (portsok == 0) {
	RPC_PDEBUG_L3("packet does not originate from a portmapper [%u]. [skip]\n",
		      ntohs(ct->tuplehash[!dir].tuple.dst.u.all));
	return RPC_PKG_NOMATCH;
      }

      /* OK: we are interested in this packet
       * Do a length check for the TCP payload
       *  A RPC reply via TCP (initial RM fragment) 
       *  consists at least of the following data:
       *  04 bytes RM header
       *  08 bytes RPC reply header
       *       04 bytes XID
       *       04 bytes MsgType
       *  16 bytes RPC reply body (mandatory prefix)
       *       04 bytes ReplyState
       *       08 bytes Verifier
       *       04 bytes AcceptState
       *
       *  In sum this is 28 bytes
       *
       *  The rest depends on the program and procedure number. E.g.
       *  - portmapper (100000)/GETPORT (3):  4 bytes
       *             04 bytes AnswerPort
       *  - portmapper (100000)/DUMP    (4): arbitrary (depends on the
       *                                     number of registered programs)
       *  - albd       (390512)/GETPORT (3): 40 bytes
       */

      /* 
       * We cannot check here for the exact length!
       */
      if ((datalen - (tcp->doff * 4)) < 28) {
	if(rpcinfo->strict == 1) {
	  *hotdrop = 1;
	  RPC_PDEBUG_L2("packet length is not plausible for RPC reply via TCP. [drop]\n");
	} else {
	  RPC_PDEBUG_L2("packet length is not plausible for RPC reply via TCP. [skip]\n");
	}
	return RPC_PKG_NOMATCH;
      }
      RPC_PDEBUG_L2("packet length is plausible. [cont]\n");
                
      /* Position the data pointer to the begin of the RPC data.
       * In case of TCP this is the 4 byte RM fragment header.
       * We provide the function check_rpc_packet_reply with the RM header
       * since we need its information for the processing of the DUMP replies.
       *
       * We have to skip the TCP header and position pointer to start
       * of the RM header
       */
      data = (const u_int32_t *)tcp + tcp->doff;

      /* Check the RPC data */
      tval = check_rpc_packet_reply_tcp(data, matchinfo, hotdrop,
					dir, ct, offset,
					datalen - (tcp->doff * 4),
					ntohl(tcp->seq));

      return tval;
      break;
                
    } /* switch (ct->tuplehash[!dir].tuple.dst.protonum) */

    RPC_PWARN("transport protocol=%u, is not supported [skip]\n",
	      ct->tuplehash[!dir].tuple.dst.protonum);
    return RPC_PKG_NOMATCH;
  }
}


static int checkentry(const char *tablename, const struct ipt_ip *ip, void *matchinfo,
                      unsigned int matchsize, unsigned int hook_mask)
{
  if (hook_mask
      & ~((1 << NF_IP_PRE_ROUTING) | (1 << NF_IP_FORWARD) | (1 << NF_IP_POST_ROUTING)
	  | (1 << NF_IP_LOCAL_IN) | (1 << NF_IP_LOCAL_OUT))) {
    printk("ipt_rpc: only valid for PRE_ROUTING, FORWARD, POST_ROUTING,\n");
    printk("         LOCAL_IN and/or LOCAL_OUT targets.\n");
    return 0;
  }

  if (matchsize != IPT_ALIGN(sizeof(struct ipt_rpc_info)))
    return 0;

  return 1;
}


static struct ipt_match rpc_match = { { NULL, NULL }, "rpc",
                                      &match, &checkentry, NULL,
                                      THIS_MODULE };


static int __init init(void)
{
  int port;

  /* If no port given, default to standard RPC port */
  if (ports[0] == 0)
    ports[0] = RPC_PORT;

  RPC_PINFO("registering match [%s] for;\n", rpc_match.name);
  for (port = 0; (port < MAX_PORTS) && ports[port]; port++) {
    RPC_PINFO("  port %i (UDP|TCP);\n", ports[port]);
    ports_n_c++;
  }

  /* Currently, we did not register our helper structure (struct ip_ct_rpc_master)
   * in union ip_conntrack_help (file ip_conntrack.h).
   * However, we check whether our helper structure needs less or equal size than
   * the officialy registered helper sturcture ip_ct_ftp_master.
   */

  RPC_PDEBUG_L3("size ip_ct_ftp_master=%d   ip_ct_rpc_master=%d \n",
		sizeof(struct ip_ct_ftp_master),
		sizeof(struct ip_ct_rpc_master));

  if (sizeof(struct ip_ct_ftp_master) < sizeof(struct ip_ct_rpc_master)) {
    RPC_PERROR("(ERROR) struct ip_ct_rpc_master too large; giving up\n");
    return -EBUSY;
  }
        
  return ipt_register_match(&rpc_match);
}


static void fini(void)
{
  RPC_PINFO("unregistering match\n");
  ipt_unregister_match(&rpc_match);
}


module_init(init);
module_exit(fini);

